"""
ReadLinkEMtab - command ``readlink -e /etc/mtab``
=================================================

The ``readlink -e /etc/mtab`` command provides information about
the path of ``mtab`` file.

Sample content from command ``readlink -e /etc/mtab`` is::
    /proc/4578/mounts

Examples:
    >>> mtab.path
    '/proc/4578/mounts'
"""

from insights.specs import Specs
from insights.parsers import SkipException
from insights import parser, CommandParser


@parser(Specs.readlink_e_etc_mtab)
class ReadLinkEMtab(CommandParser):
    """Class for command: readlink -e /etc/mtab"""
    def parse_content(self, content):
        if content is None or len(content) == 0:
            raise SkipException("No Data from command: readlink -e /etc/mtab")

        for line in content:
            self._path = line  # use the last line

    @property
    def path(self):
        """Returns real file path from command"""
        return self._path



"""
SambaConfig - file ``/etc/samba/smb.conf``
==========================================

This parser reads the Samba configuration file ``/etc/samba/smb.conf``, which
is in standard .ini format, with a couple of notable features:

* Samba ignores spaces at the start of options, which the ConfigParser class
  normally does not. This spacing is stripped by this parser.
* Samba likewise ignores spaces in section heading names.
* Samba allows the same section to be defined multiple times, with the
  options therein being merged as if they were one section.
* Samba allows options to be declared before the first section marker.
  This parser puts these options in a `global` section.
* Samba treats ';' as a comment prefix, similar to '#'.

Sample configuration file::

    # This is the main Samba configuration file. You should read the
    # smb.conf(5) manual page in order to understand the options listed
    #...
    #======================= Global Settings =====================================

    [global]
        workgroup = MYGROUP
        server string = Samba Server Version %v
        max log size = 50

    [homes]
        comment = Home Directories
        browseable = no
        writable = yes
    ;   valid users = %S
    ;   valid users = MYDOMAIN\%S

    [printers]
        comment = All Printers
        path = /var/spool/samba
        browseable = no
        guest ok = no
        writable = no
        printable = yes

    # A publicly accessible directory, but read only, except for people in
    # the "staff" group
    [public]
       comment = Public Stuff
       path = /home/samba
       public = yes
       writable = yes
       printable = no
       write list = +staff

Examples:

    >>> type(conf)
    <class 'insights.parsers.samba.SambaConfig'>
    >>> sorted(conf.sections()) == [u'global', u'homes', u'printers', u'public']
    True
    >>> global_options = conf.items('global')  # get a section as a dictionary
    >>> type(global_options) == type({})
    True
    >>> conf.get('public', 'comment') == u'Public Stuff'  # Accessor for section and option
    True
    >>> conf.getboolean('public', 'writable')  # Type conversion, but no default
    True
    >>> conf.getint('global', 'max log size')  # Same for integer conversion
    50

"""
import re

from . import ParseException
from .. import add_filter, IniConfigFile, parser
from insights.specs import Specs

add_filter(Specs.samba, ["["])

add_filter(Specs.testparm_s, ["["])
add_filter(Specs.testparm_s, ["Server role:"])

add_filter(Specs.testparm_v_s, ["["])
add_filter(Specs.testparm_v_s, ["Server role:"])


@parser(Specs.samba)
class SambaConfig(IniConfigFile):
    """
    This parser reads the Samba configuration file ``/etc/samba/smb.conf``.

    Note: It is needed for better resolution descriptions when it is necessary to know what exactly
    is in the configuration file. For generic tasks use ``SambaConfigs`` or ``SambaConfigsAll``
    instead.
    """
    def parse_content(self, content):
        # smb.conf is special from other ini files in the property that
        # whatever is before the first section (before the first section)
        # belongs to the [global] section. Therefore, the [global] section is
        # appended right at the beginning so that everything that would be
        # parsed as outside section belongs to [global].
        # Python 2.7 RawConfigParser automatically merges multiple instances
        # of the same section.  (And if that ever changes, test_samba.py will
        # catch it.)
        lstripped = ["[global]"] + [line.lstrip() for line in content]

        super(SambaConfig, self).parse_content(lstripped)

        # Create a new instance of the same dict type used by the underlying
        # RawConfigParser.
        new_dict = self.data._dict()
        # Transform the section names so that whitespace around is stripped
        # and they are lowercase. smb.conf is special in the property that
        # section names and option names are case-insensitive and treated
        # like lower-case.
        for old_key, old_section in self.data._sections.items():
            new_key = old_key.strip().lower()
            if new_key not in new_dict:
                new_dict[new_key] = self.data._dict()
            # Merge same-named sections just as samba's `testparm` does.
            new_dict[new_key].update(old_section)
        self.data._sections = new_dict


@parser(Specs.testparm_s)
class SambaConfigs(SambaConfig):
    """
    This parser reads the Samba configuration from command `testparm -s` which is more reliable
    than parsing the config file, as it includes configuration in internal registry. It also
    includes server role.

    Note: This is the most suitable parser when only user changes to the configuration are important
    for the detection logic, i.e. misconfiguration.

    Attributes:
        server_role (string): Server role as reported by the command.
    """
    def parse_content(self, content):
        # Parse server role
        for line in content:
            r = re.search(r"Server role:\s+(\S+)", line)
            if r:
                self.server_role = r.group(1)
                break
        else:
            raise ParseException("Server role not found.")

        super(SambaConfigs, self).parse_content(content)


@parser(Specs.testparm_v_s)
class SambaConfigsAll(SambaConfigs):
    """
    This parser reads the Samba configuration from command `testparm -v -s` which is more reliable
    than parsing the config file, as it includes configuration in internal registry. It also
    includes all default values and server role.

    Note: This parser is needed for cases when active value of specific option is needed for the
    detection logic, irrespective of its origin from user changes or defaults, i.e. security
    vulnerabilities.

    Attributes:
        server_role (string): Server role as reported by the command.
    """
    pass



from insights import rule, make_fail, make_pass, make_info


@rule()
def report():
    return make_fail("SOME_FAIL", foo="bar")


@rule()
def report2():
    return make_pass("SOME_PASS", foo="bar")


@rule()
def report3():
    return make_info("SOME_INFO", foo="bar")



"""
AuditdConf - file ``/etc/audit/auditd.conf``
============================================

The auditd.conf file is a standard key = value file with hash comments.
Active settings are provided using the `get_active_settings_value` method or
by using the dictionary `contains` functionality.

Example:

    >>> conf = shared[AuditdConf]
    >>> conf.get_active_setting_value('log_group')
    'root'
    >>> 'log_file' in conf
    True
"""

from .. import Parser, parser, get_active_lines
from ..parsers import split_kv_pairs
from insights.specs import Specs


@parser(Specs.auditd_conf)
class AuditdConf(Parser):
    """
    A parser for accessing /etc/audit/auditd.conf.
    """

    def __init__(self, *args, **kwargs):
        self.active_lines_unparsed = []
        self.active_settings = {}
        super(AuditdConf, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        """
        Main parsing class method which stores all interesting data from the content.

        Args:
            content (context.content): Parser context content
        """
        self.active_lines_unparsed = get_active_lines(content)
        #  (man page specifies that a line must contain "=")
        self.active_settings = split_kv_pairs(content, use_partition=False)

    def get_active_setting_value(self, setting_name):
        """
        Access active setting value by setting name.

        Args:
            setting_name (string): Setting name
        """
        return self.active_settings[setting_name]



from insights.core.taglang import parse


def test_simple():
    pred = parse("security")
    assert pred(["security"])
    assert not pred(["logging"])


def test_regex():
    """
    Regexes start with a forward slash (/). If they must contain spaces,
    enclose the regex after the forward slash in quotes.
    """
    pred = parse("/net")
    assert pred(["networking"])
    assert pred(["tenet"])
    assert not pred(["security"])

    pred = parse("/^net")
    assert pred(["networking"])
    assert not pred(["tenet"])
    assert not pred(["security"])

    pred = parse("/'net work'")
    assert pred(["net work"])
    assert not pred(["networking"])
    assert not pred(["tenet"])
    assert not pred(["security"])


def test_negation():
    pred = parse("!security")
    assert not pred(["security", "logging"])
    assert not pred(["security"])
    assert pred(["logging"])
    assert pred(["jboss"])
    assert pred([])


def test_conjunction():
    pred = parse("security & logging")
    assert pred(["security", "logging"])
    assert not pred(["security"])
    assert not pred(["logging"])


def test_disjunction():
    pred = parse("security | logging")
    assert pred(["security", "logging"])
    assert pred(["security"])
    assert pred(["logging"])
    assert not pred(["jboss"])

    # , is a synonym for |
    pred = parse("security, logging")
    assert pred(["security", "logging"])
    assert pred(["security"])
    assert pred(["logging"])
    assert not pred(["jboss"])


def test_precedence():
    """ ! binds more strongly than &, which binds more strongly than | """
    pred = parse("security | logging & !jboss")
    assert pred(["security"])
    assert pred(["logging"])
    assert not pred(["logging", "jboss"])
    assert not pred(["apache", "jboss"])


def test_grouping():
    pred = parse("(security | logging) & jboss")
    assert pred(["security", "jboss"])
    assert pred(["logging", "jboss"])
    assert not pred(["security", "logging"])
    assert not pred(["security"])
    assert not pred(["logging"])
    assert not pred(["jboss"])


def test_regex_in_group():
    """
    A regex at the end of a group must have a space before the closing paren
    or the paren will be considered part of the regex.
    """
    pred = parse("(security | /logging ) & jboss")
    assert pred(["security", "jboss"])
    assert pred(["logging", "jboss"])
    assert not pred(["security", "logging"])
    assert not pred(["security"])
    assert not pred(["logging"])
    assert not pred(["jboss"])


def test_quoted_string():
    # notice the entire string is quoted, so it's a single tag instead of
    # multiple tags connected by operators.
    pred = parse("'(security | /logging ) & jboss'")
    assert pred(['(security | /logging ) & jboss'])
    assert not pred(["security", "jboss"])
    assert not pred(["logging", "jboss"])



"""
CephVersion - command ``ceph -v``
=================================

This module provides plugins access to the Ceph version information gathered from
the ``ceph -v`` command. This module parses the community version to the Red Hat
release version.

The Red Hat Ceph Storage releases and corresponding Ceph package releases are
documented in https://access.redhat.com/solutions/2045583

"""

import re
from insights import parser, CommandParser
from insights.parsers import SkipException
from insights.specs import Specs
from insights.util import deprecated

# TODO: the following metrics need update timely per:
# - https://access.redhat.com/solutions/2045583
# - https://access.redhat.com/articles/1372203
community_to_release_map = {
    "0.80.8-5": {'version': "1.2.3", 'major': '1.2', 'minor': '3', 'downstream_release': 'NA'},
    "0.94.1-15": {'version': "1.3", 'major': '1.3', 'minor': '0', 'downstream_release': 'NA'},
    "0.94.3-3": {'version': "1.3.1", 'major': '1.3', 'minor': '1', 'downstream_release': 'NA'},
    "0.94.5-9": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'NA'},
    "0.94.5-12": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.5-13": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.5-14": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.5-15": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'async'},
    "0.94.9-3": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'NA'},
    "0.94.9-8": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'async'},
    "0.94.9-9": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'async 2'},
    "0.94.10-2": {'version': "1.3.4", 'major': '1.3', 'minor': '4', 'downstream_release': 'NA'},
    "10.2.2-38": {'version': "2.0", 'major': '2', 'minor': '0', 'downstream_release': '0'},
    "10.2.2-41": {'version': "2.0", 'major': '2', 'minor': '0', 'downstream_release': 'async'},
    "10.2.3-13": {'version': "2.1", 'major': '2', 'minor': '1', 'downstream_release': '0'},
    "10.2.3-17": {'version': "2.1", 'major': '2', 'minor': '1', 'downstream_release': 'async'},
    "10.2.5-37": {'version': "2.2", 'major': '2', 'minor': '2', 'downstream_release': '0'},
    "10.2.7-27": {'version': "2.3", 'major': '2', 'minor': '3', 'downstream_release': '0'},
    "10.2.7-28": {'version': "2.3", 'major': '2', 'minor': '3', 'downstream_release': 'async'},
    "10.2.7-32": {'version': "2.4", 'major': '2', 'minor': '4', 'downstream_release': '0'},
    "10.2.7-48": {'version': "2.4", 'major': '2', 'minor': '4', 'downstream_release': 'async'},
    "10.2.10-16": {'version': "2.5", 'major': '2', 'minor': '5', 'downstream_release': '0'},
    "10.2.10-17": {'version': "2.5", 'major': '2', 'minor': '5', 'downstream_release': 'async'},
    "10.2.10-28": {'version': "2.5.1", 'major': '2', 'minor': '5', 'downstream_release': '1'},
    "10.2.10-40": {'version': "2.5.2", 'major': '2', 'minor': '5', 'downstream_release': '2'},
    "10.2.10-43": {'version': "2.5.3", 'major': '2', 'minor': '5', 'downstream_release': '3'},
    "10.2.10-49": {'version': "2.5.4", 'major': '2', 'minor': '5', 'downstream_release': '4'},
    "10.2.10-51": {'version': "2.5.5", 'major': '2', 'minor': '5', 'downstream_release': '5', 'els': True},
    "12.2.1-40": {'version': "3.0", 'major': '3', 'minor': '0', 'downstream_release': '0'},
    "12.2.1-45": {'version': "3.0", 'major': '3', 'minor': '0', 'downstream_release': '1'},
    "12.2.1-46": {'version': "3.0", 'major': '3', 'minor': '0', 'downstream_release': '1 CVE'},
    "12.2.4-6": {'version': "3.0.2", 'major': '3', 'minor': '0', 'downstream_release': '2'},
    "12.2.4-10": {'version': "3.0.3", 'major': '3', 'minor': '0', 'downstream_release': '3'},
    "12.2.4-30": {'version': "3.0.4", 'major': '3', 'minor': '0', 'downstream_release': '4'},
    "12.2.4-42": {'version': "3.0.5", 'major': '3', 'minor': '0', 'downstream_release': '5'},
    "12.2.5-42": {'version': "3.1", 'major': '3', 'minor': '1', 'downstream_release': '0'},
    "12.2.5-59": {'version': "3.1.1", 'major': '3', 'minor': '1', 'downstream_release': '1'},
    "12.2.8-52": {'version': "3.2", 'major': '3', 'minor': '2', 'downstream_release': '0'},
    "12.2.8-89": {'version': "3.2.1", 'major': '3', 'minor': '2', 'downstream_release': '1'},
    "12.2.8-128": {'version': "3.2.2", 'major': '3', 'minor': '2', 'downstream_release': '2'},
    "12.2.12-45": {'version': "3.3", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-48": {'version': "3.3", 'major': '3', 'minor': '3', 'downstream_release': 'async'},
    "12.2.12-74": {'version': "3.3.1", 'major': '3', 'minor': '3', 'downstream_release': '1'},
    "12.2.12-79": {'version': "3.3.1", 'major': '3', 'minor': '3', 'downstream_release': 'async'},
    "12.2.12-84": {'version': "3.3.2", 'major': '3', 'minor': '3', 'downstream_release': '2'},
    "12.2.12-101": {'version': "3.3.4", 'major': '3', 'minor': '3', 'downstream_release': '4'},
    "12.2.12-115": {'version': "3.3.5", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-124": {'version': "3.3.6", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "12.2.12-127": {'version': "3.3.6", 'major': '3', 'minor': '3', 'downstream_release': '0'},
    "14.2.4-125": {'version': "4.0", 'major': '4', 'minor': '0', 'downstream_release': '0'},
    "14.2.4-51": {'version': "4.0", 'major': '4', 'minor': '0', 'downstream_release': '0'},
    "14.2.8-50": {'version': "4.1", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-59": {'version': "4.1", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-81": {'version': "4.1.1", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-89": {'version': "4.1.1", 'major': '4', 'minor': '1', 'downstream_release': 'async'},
    "14.2.8-91": {'version': "4.1.1", 'major': '4', 'minor': '1', 'downstream_release': 'async 2'},
    "14.2.8-111": {'version': "4.1.2", 'major': '4', 'minor': '1', 'downstream_release': '0'},
    "14.2.8-115": {'version': "4.1.3", 'major': '4', 'minor': '1', 'downstream_release': '0'},
}


@parser(Specs.ceph_v)
class CephVersion(CommandParser):
    """
    Class for parsing the output of command ``ceph -v``.

    Typical output of the ``ceph -v`` command is::

        ceph version 0.94.9-9.el7cp (b83334e01379f267fb2f9ce729d74a0a8fa1e92c)

    Attributes:
        version (str): The Red Hat release version
        major (str): The major version of Red Hat release version
        minor (str): The minor version of Red Hat release version
        is_els (boolean): If the verion in 'Extended life cycle support (ELS) add-on' phase
        downstream_release (str): The downstream release info
        upstream_version (dict): The detailed upstream version info with the
            following keys `release (int)`, `major (int)` and `minor (int)`.

    Example:
        >>> ceph_v.version
        '1.3.3'
        >>> ceph_v.major
        '1.3'
        >>> ceph_v.minor
        '3'
        >>> ceph_v.is_els
        False
    """

    def parse_content(self, content):
        # Parse Ceph Version Content and get Release, Major, Minor number
        if not content:
            raise SkipException("Empty Ceph Version Line", content)

        ceph_version_line = content[-1]
        # re search pattern
        pattern_community = r'((\d{1,2})\.(\d{1,2})\.((\d{1,2})|x))((\-(\d+)))'
        community_version_mo = re.search(pattern_community, str(ceph_version_line), 0)
        if not community_version_mo:
            raise SkipException("Wrong Format Ceph Version", content)

        community_version = community_version_mo.group(0)
        release_data = community_to_release_map.get(community_version, None)
        if not release_data:
            raise SkipException("No Mapping Release Version. Ceph Release Number is Null", content)

        self.version = release_data['version']
        self.major = release_data['major']
        self.minor = release_data['minor']
        self.is_els = release_data.get('els', False)
        self.downstream_release = release_data['downstream_release']
        self.upstream_version = {
            "release": int(community_version_mo.group(2)),
            "major": int(community_version_mo.group(3)),
            "minor": int(community_version_mo.group(4))
        }


class CephVersionError(Exception):
    """
    .. note::
        This class is deprecated, please use :py:class:`insights.parsers.SkipException` instead.
    """

    def __init__(self, *args, **kwargs):
        deprecated(CephVersionError, "Use SkipException instead.")
        super(CephVersionError, self).__init__(*args, **kwargs)



# -*- coding: UTF-8 -*-
from insights.core import AttributeDict

DICT = {'dmode': '0500', 'relatime': True, 'uid': '0',
        'iocharset': 'utf8', 'gid': '0', 'mode': '0400', 'ro': True,
        'nosuid': True, 'uhelper': 'udisks2', 'nodev': True}


def test_attribute_dict():
    d_obj = AttributeDict(DICT)
    assert d_obj['dmode'] == '0500'
    assert d_obj.get('relatime') is True
    assert d_obj.nosuid is True
    assert 'uid' in d_obj
    assert d_obj.get('mode', None) == '0400'
    assert d_obj.get('xmode', None) is None
    for d, v in d_obj.items():
        if d.startswith('nod'):
            assert v is True



"""
InsightsClientConf - file ``/etc/insights-client/insights-client.conf``
=======================================================================

"""
from insights import IniConfigFile, parser, add_filter
from insights.specs import Specs

add_filter(Specs.insights_client_conf, "[")


@parser(Specs.insights_client_conf)
class InsightsClientConf(IniConfigFile):
    """
    This class provides parsing for the file ``/etc/insights-client/insights-client.conf``.

    Sample input data is in the format::

        [insights-client]
        auto_update=False

    See the :class:`insights.core.IniConfigFile` class for examples.
    """
    pass



"""
QPID statistics - command ``qpid-stat``
=======================================

This module contains parsers that check the QPID daemon statistics.  qpidd is
used by Satellite for communication between clients, capsules and servers.

Parsers provided by this module are:

QpidStatQ - command ``/usr/bin/qpid-stat -q --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b amqps://localhost:5671``
----------------------------------------------------------------------------------------------------------------------------------

QpidStatU - command ``/usr/bin/qpid-stat -u --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b amqps://localhost:5671``
----------------------------------------------------------------------------------------------------------------------------------

QpidStatG - command ``/usr/bin/qpid-stat -g --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b amqps://localhost:5671``
----------------------------------------------------------------------------------------------------------------------------------

"""

from insights import CommandParser, get_active_lines, parser
from insights.parsers import ParseException, keyword_search, parse_fixed_table
from insights.specs import Specs


class QpidStat(CommandParser):
    """Base class for parsing QpidStat command."""
    def __init__(self, context):
        self.data = []
        self.by_queue = {}
        super(QpidStat, self).__init__(context)

    def _check_content(self, content):
        self._content = get_active_lines(content, '=======')
        if len(self._content) <= 1:
            raise ParseException("Input content is empty or there is no useful parsed data.")

    def _parse_data(self, heading_ignore):
        self.data.extend(parse_fixed_table(self._content, heading_ignore=heading_ignore))

    def _parse_by_key(self, key):
        self.by_queue.update(dict(
            (q[key], q)
            for q in self.data
            if key in q
        ))

    def parse_content(self, content):
        self._check_content(content)
        self._parse_data('queue')
        self._parse_by_key('queue')
        del self._content

    def __iter__(self):
        return iter(self.data)

    def search(self, **kwargs):
        """
        Search for rows in the data matching keywords in the search.

        This method uses the :py:func:`insights.parsers.keyword_search`
        function - see its documentation for a complete description of its
        keyword recognition capabilities.

        Arguments:
            **kwargs: Key-value pairs of search parameters.

        Returns:
            (list): A list of subscriptions that matched the search criteria.

        """
        return keyword_search(self.data, **kwargs)


@parser(Specs.qpid_stat_q)
class QpidStatQ(QpidStat):
    """
    This parser reads the output of the command ``qpid-stat -q
    --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b
    amqps://localhost:5671``

    Sample output::

        Queues
          queue                                                                      dur  autoDel  excl  msg   msgIn  msgOut  bytes  bytesIn  bytesOut  cons  bind
          ==========================================================================================================================================================
          00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0                                        Y        Y        0     2      2       0    486      486         1     2
          0f7f1a3d-daff-42a6-a994-29050a2eabde:1.0                                        Y        Y        0     8      8       0   4.88k    4.88k        1     2

    Attributes:
        data (list of dict): A list of dictionaries with the key-value data
            from the table.
        by_queue (dict of dict): A dictionary of the same data dictionaries
            stored by queue ID.

    Examples:
        >>> type(qpid_stat_q)
        <class 'insights.parsers.qpid_stat.QpidStatQ'>
        >>> type(qpid_stat_q.data) == type([])  # Queue data stored as it appears
        True
        >>> type(qpid_stat_q.data[0]) == type({}) # Each row is a dictionary formed from the table
        True
        >>> qpid_stat_q.data[0]['queue']
        '00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0'
        >>> qpid_stat_q.data[0]['dur']  # Blank columns are empty strings
        ''
        >>> qpid_stat_q.data[0]['autoDel']  # Flags are left as strings
        'Y'
        >>> qpid_stat_q.data[0]['msgOut']  # Numbers are left as strings
        '2'
        >>> qpid_stat_q.data[1]['bytesOut']  # No byte measure conversion
        '4.88k'
        >>> type(qpid_stat_q.by_queue) == type({}) # Dictionary lookup by queue ID
        True
        >>> qpid_stat_q.by_queue['00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0'] == qpid_stat_q.data[0]
        True
        >>> total_messages_in = 0
        >>> for queue in qpid_stat_q:  # Can be used as an iterator
        ...     total_messages_in += int(queue['msgIn'])
        ...
        >>> total_messages_in
        10
        >>> qpid_stat_q.search(queue__contains=':2.0')  # Keyword search
        []
    """
    pass


@parser(Specs.qpid_stat_u)
class QpidStatU(QpidStat):

    """
    This parser reads the output of the command ``qpid-stat -u
    --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b
    amqps://localhost:5671``

    Sample output::

        Subscriptions
          subscr               queue                                                                      conn                                    procName          procId  browse  acked  excl  creditMode  delivered  sessUnacked
          ===========================================================================================================================================================================================================================
          0                    00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0                                   qpid.10.20.1.10:5671-10.20.1.10:33787   celery            21409                        CREDIT      2          0
          0                    pulp.agent.c6a430bc-5ec7-42f8-99ce-f320ed0b9113                            qpid.10.20.1.10:5671-10.30.0.148:57423  goferd            32227           Y            CREDIT      0          0
          1                    server.example.com:event                                                   qpid.10.20.1.10:5671-10.20.1.10:33848   Qpid Java Client  21066           Y      Y     WINDOW      2,623      0
          0                    celeryev.4c77bd03-1cde-49eb-bdc0-b7c38f9ff93d                              qpid.10.20.1.10:5671-10.20.1.10:33777   celery            21356           Y            CREDIT      363,228    0
          1                    celery                                                                     qpid.10.20.1.10:5671-10.20.1.10:33786   celery            21409           Y            CREDIT      5          0

    Attributes:
        data (list of dict): A list of dictionaries with the key-value data
            from the table.
        by_queue (dict of dict): A dictionary of the same data dictionaries
            stored by queue ID.

    Examples:
        >>> type(qpid_stat_u)
        <class 'insights.parsers.qpid_stat.QpidStatU'>
        >>> type(qpid_stat_u.data) == type([]) # Subscription data stored as it appears
        True
        >>> type(qpid_stat_u.data[0]) == type({}) # Each row is a dictionary formed from the table
        True
        >>> qpid_stat_u.data[0]['queue']
        '00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0'
        >>> qpid_stat_u.data[0]['browse']  # Blank columns are empty strings
        ''
        >>> qpid_stat_u.data[1]['acked']  # Flags are left as strings
        'Y'
        >>> qpid_stat_u.data[1]['subscr']  # Numbers are left as strings
        '0'
        >>> qpid_stat_u.data[2]['delivered']  # Beware the commas
        '2,623'
        >>> type(qpid_stat_u.by_queue) == type({}) # Dictionary lookup by queue ID
        True
        >>> qpid_stat_u.by_queue['celery'] == qpid_stat_u.data[4]
        True
        >>> total_celery_queues = 0
        >>> for subscr in qpid_stat_u:  # Can be used as an iterator
        ...     if subscr['procName'] == 'celery':
        ...         total_celery_queues += 1
        ...
        >>> total_celery_queues
        3
        >>> event_queues = qpid_stat_u.search(queue__contains=':event')  # Keyword search
        >>> type(event_queues) == type([])
        True
        >>> len(event_queues)
        1
        >>> event_queues[0] == qpid_stat_u.data[2]  # List contains matching items
        True
    """

    def parse_content(self, content):
        self._check_content(content)
        self._parse_data('subscr')
        self._parse_by_key('queue')
        del self._content


@parser(Specs.qpid_stat_g)
class QpidStatG(QpidStat):

    """
    This parser reads the output of the command ``qpid-stat -g
    --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b
    amqps://localhost:5671``

    Sample output::

        Broker Summary:
          uptime           cluster       connections  sessions  exchanges  queues
          =========================================================================
          96d 23h 50m 41s  <standalone>  23           37        14         33

        Aggregate Broker Statistics:
          Statistic                   Messages    Bytes
          ========================================================
          queue-depth                 0           0
          total-enqueues              1,726,798   42,589,932,236
          total-dequeues              1,726,798   42,589,932,236
          persistent-enqueues         28,725      23,889,836
          persistent-dequeues         28,725      23,889,836
          transactional-enqueues      0           0
          transactional-dequeues      0           0
          flow-to-disk-depth          0           0
          flow-to-disk-enqueues       0           0
          flow-to-disk-dequeues       0           0
          acquires                    1,726,798
          releases                    0
          discards-no-route           41,163,896

    Attributes:
        data (list of dict): A list of dictionaries with the key-value data
            from the table.
        by_queue (dict of dict): A dictionary of the same data dictionaries
            stored by cluster name or queue name.

    Examples:
        >>> type(qpid_stat_g)
        <class 'insights.parsers.qpid_stat.QpidStatG'>
        >>> type(qpid_stat_g.data) == type([])
        True
        >>> type(qpid_stat_g.data[0]) == type({}) # Each row is a dictionary formed from the table
        True
        >>> qpid_stat_g.data[0]['uptime']
        '97d 0h 16m 24s'
        >>> qpid_stat_g.data[0]['cluster']
        '<standalone>'
        >>> qpid_stat_g.data[1]['Statistic']
        'queue-depth'
        >>> qpid_stat_g.data[1]['Messages']
        '0'
        >>> qpid_stat_g.data[11]['Bytes']
        ''
        >>> type(qpid_stat_g.by_queue) == type({}) # Dictionary lookup by queue ID
        True
        >>> qpid_stat_g.by_queue['queue-depth'] == qpid_stat_g.data[1]
        True
        >>> enqueues = qpid_stat_g.search(Statistic__contains='enqueues')  # Keyword search
        >>> type(enqueues) == type([])
        True
        >>> len(enqueues)
        4
        >>> enqueues[0] == qpid_stat_g.data[2]  # List contains matching items
        True
    """
    def _split_content(self):
        index = [i for i, l in enumerate(self._content) if l.startswith("Aggregate Broker Statistics")]
        if not index:
            raise ParseException("Incorrect Content without \"Aggregate Broker Statistics\" in ")
        self._index = index[0]

    def parse_content(self, content):
        self._check_content(content)
        self._split_content()
        self.__content = self._content
        self._content = self.__content[0:self._index]
        self._parse_data('uptime')
        self._content = self.__content[self._index:]
        self._parse_data('Statistic')
        self._parse_by_key('cluster')
        self._parse_by_key('Statistic')
        del self._index
        del self.__content
        del self._content



import pytest
import doctest
from insights.parsers import azure_instance_type
from insights.parsers.azure_instance_type import AzureInstanceType
from insights.tests import context_wrap
from insights.parsers import SkipException, ParseException
from insights.core.plugins import ContentException

AZURE_TYPE_1 = "Standard_L32s"
AZURE_TYPE_2 = "Standard_NV48s_v3"
AZURE_TYPE_3 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
Standard_NV48s_v3
"""
AZURE_TYPE_DOC = "Standard_L64s_v2"
AZURE_TYPE_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
AZURE_TYPE_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
AZURE_TYPE_AB_3 = """
curl: (28) connect() timed out!
""".strip()
AZURE_TYPE_AB_4 = """
.micro
""".strip()
AZURE_TYPE_AB_5 = """
No module named insights.tools
""".strip()


def test_azure_instance_type_ab_other():
    with pytest.raises(SkipException):
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_1))

    with pytest.raises(SkipException):
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_2))

    with pytest.raises(SkipException):
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_3))

    with pytest.raises(ParseException) as pe:
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_4))
        assert 'Unrecognized type' in str(pe)

    with pytest.raises(ContentException) as pe:
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_5))


def test_azure_instance_type_ab_empty():
    with pytest.raises(SkipException):
        AzureInstanceType(context_wrap(''))


def test_azure_instance_type():
    azure = AzureInstanceType(context_wrap(AZURE_TYPE_1))
    assert azure.type == "Standard"
    assert azure.size == "L32s"
    assert azure.version is None
    assert azure.raw == "Standard_L32s"

    azure = AzureInstanceType(context_wrap(AZURE_TYPE_2))
    assert azure.type == "Standard"
    assert azure.size == "NV48s"
    assert azure.version == "v3"
    assert azure.raw == "Standard_NV48s_v3"
    assert "NV48s" in str(azure)


def test_azure_instance_type_stats():
    azure = AzureInstanceType(context_wrap(AZURE_TYPE_3))
    assert azure.type == "Standard"
    assert azure.size == "NV48s"
    assert azure.version == "v3"
    assert azure.raw == "Standard_NV48s_v3"


def test_doc_examples():
    env = {
            'azure_inst': AzureInstanceType(context_wrap(AZURE_TYPE_DOC))
          }
    failed, total = doctest.testmod(azure_instance_type, globs=env)
    assert failed == 0



from insights.parsers import gfs2_file_system_block_size
from insights.tests import context_wrap
from insights.parsers import SkipException
import pytest
import doctest

BLOCK_SIZE_OUTPUT = """
4096
""".strip()

BLOCK_SIZE_OUTPUT_2 = """
abc
""".strip()

BLOCK_SIZE_OUTPUT_3 = """
512
""".strip()

BLOCK_SIZE_OUTPUT_4 = """
4096
512
""".strip()

BLOCK_SIZE_OUTPUT_5 = """
stat: missing operand
Try 'stat --help' for more information.
""".strip()


def test_exp():
    with pytest.raises(SkipException):
        gfs2_file_system_block_size.GFS2FileSystemBlockSize(context_wrap(BLOCK_SIZE_OUTPUT_2))
    with pytest.raises(SkipException):
        gfs2_file_system_block_size.GFS2FileSystemBlockSize(context_wrap(BLOCK_SIZE_OUTPUT_4))
    with pytest.raises(SkipException):
        gfs2_file_system_block_size.GFS2FileSystemBlockSize(context_wrap(BLOCK_SIZE_OUTPUT_5))


def test_doc():
    env = {
            'gfs2_mp': gfs2_file_system_block_size.GFS2FileSystemBlockSize(context_wrap(BLOCK_SIZE_OUTPUT))
          }
    failed, total = doctest.testmod(gfs2_file_system_block_size, globs=env)
    assert failed == 0


def test_other_size():
    gfs2 = gfs2_file_system_block_size.GFS2FileSystemBlockSize(context_wrap(BLOCK_SIZE_OUTPUT_3))
    assert gfs2.block_size == 512



from insights.parsers import openshift_get_with_config, ParseException
from insights.parsers.openshift_get_with_config import OcGetClusterRoleWithConfig, OcGetClusterRoleBindingWithConfig
from insights.tests import context_wrap
import doctest
import pytest

OC_GET_CLUSTER_ROLE = """
NAME
admin
asb-access
asb-auth
basic-user
cluster-admin
cluster-debugger
cluster-reader
cluster-status
edit
hawkular-metrics-admin
management-infra-admin
namespace-viewer
registry-admin
registry-editor
registry-viewer
rolebinding-reader
sar-creator
self-access-reviewer
self-provisioner
service-catalog-controller
servicecatalog-serviceclass-viewer
storage-admin
sudoer
system:aggregate-to-admin
system:aggregate-to-edit
system:aggregate-to-view
system:auth-delegator
system:aws-cloud-provider
system:basic-user
system:build-controller
system:build-strategy-custom
system:build-strategy-docker
system:build-strategy-jenkinspipeline
system:build-strategy-source
system:certificate-signing-controller
system:certificates.k8s.io:certificatesigningrequests:nodeclient
system:certificates.k8s.io:certificatesigningrequests:selfnodeclient
system:controller:attachdetach-controller
system:controller:certificate-controller
system:controller:clusterrole-aggregation-controller
system:controller:cronjob-controller
system:controller:daemon-set-controller
system:controller:deployment-controller
system:controller:disruption-controller
system:controller:endpoint-controller
system:controller:generic-garbage-collector
system:controller:horizontal-pod-autoscaler
system:controller:job-controller
system:controller:namespace-controller
system:controller:node-controller
system:controller:persistent-volume-binder
system:controller:pod-garbage-collector
system:controller:replicaset-controller
system:controller:replication-controller
system:controller:resourcequota-controller
system:controller:route-controller
system:controller:service-account-controller
system:controller:service-controller
system:controller:statefulset-controller
system:controller:ttl-controller
system:daemonset-controller
system:deployer
system:deployment-controller
system:deploymentconfig-controller
system:discovery
system:disruption-controller
system:endpoint-controller
system:garbage-collector-controller
system:gc-controller
system:heapster
system:hpa-controller
system:image-auditor
system:image-builder
system:image-pruner
system:image-puller
system:image-pusher
system:image-signer
system:job-controller
system:kube-aggregator
system:kube-controller-manager
system:kube-dns
system:kube-scheduler
system:master
system:namespace-controller
system:node
system:node-admin
system:node-bootstrapper
system:node-problem-detector
system:node-proxier
system:node-reader
system:oauth-token-deleter
system:openshift:aggregate-to-admin
system:openshift:aggregate-to-edit
system:openshift:aggregate-to-view
system:openshift:controller:build-config-change-controller
system:openshift:controller:build-controller
system:openshift:controller:cluster-quota-reconciliation-controller
system:openshift:controller:deployer-controller
system:openshift:controller:deploymentconfig-controller
system:openshift:controller:horizontal-pod-autoscaler
system:openshift:controller:image-import-controller
system:openshift:controller:image-trigger-controller
system:openshift:controller:origin-namespace-controller
system:openshift:controller:pv-recycler-controller
system:openshift:controller:resourcequota-controller
system:openshift:controller:sdn-controller
system:openshift:controller:service-ingress-ip-controller
system:openshift:controller:service-serving-cert-controller
system:openshift:controller:serviceaccount-controller
system:openshift:controller:serviceaccount-pull-secrets-controller
system:openshift:controller:template-instance-controller
system:openshift:controller:template-service-broker
system:openshift:controller:unidling-controller
system:openshift:templateservicebroker-client
system:persistent-volume-provisioner
system:registry
system:replicaset-controller
system:replication-controller
system:router
system:scope-impersonation
system:sdn-manager
system:sdn-reader
system:service-catalog:aggregate-to-admin
system:service-catalog:aggregate-to-edit
system:service-catalog:aggregate-to-view
system:statefulset-controller
system:webhook
view
""".strip()

OC_GET_CLUSTERROLEBINDING = """
NAME                                                                  ROLE                                                                   USERS                            GROUPS                                         SERVICE ACCOUNTS                                                                   SUBJECTS
admin                                                                 /admin                                                                                                                                                 openshift-infra/template-instance-controller
admin-0                                                               /admin                                                                                                                                                 kube-service-catalog/default
admin-1                                                               /admin                                                                                                                                                 openshift-ansible-service-broker/asb
asb-access                                                            /asb-access                                                                                                                                            openshift-ansible-service-broker/asb-client
asb-auth                                                              /asb-auth                                                                                                                                              openshift-ansible-service-broker/asb
auth-delegator-openshift-template-service-broker                      /system:auth-delegator                                                                                                                                 openshift-template-service-broker/apiserver
basic-users                                                           /basic-user                                                                                             system:authenticated
cluster-admin                                                         /cluster-admin                                                                                          system:masters
cluster-admin-0                                                       /cluster-admin                                                                                                                                         insights-scan/insights-scan
cluster-admins                                                        /cluster-admin
""".strip()

OC_GET_CLUSTERROLEBINDING_INVALID = """
NAME                                                                  ROLE                                                                   USERS                            GROUPS                                         SERVICE ACCOUNTS                                                                   SUBJECTS
admin
admin-0                                                               /admin                                                                                                                                                 kube-service-catalog/default
admin-1                                                               /admin                                                                                                                                                 openshift-ansible-service-broker/asb
asb-access                                                            /asb-access                                                                                                                                            openshift-ansible-service-broker/asb-client
asb-auth                                                              /asb-auth                                                                                                                                              openshift-ansible-service-broker/asb
auth-delegator-openshift-template-service-broker                      /system:auth-delegator                                                                                                                                 openshift-template-service-broker/apiserver
basic-users                                                           /basic-user                                                                                             system:authenticated
cluster-admin                                                         /cluster-admin                                                                                          system:masters
cluster-admin-0                                                       /cluster-admin                                                                                                                                         insights-scan/insights-scan
cluster-admins                                                        /cluster-admin
""".strip()

OC_GET_CLUSTERROLEBINDING_INVALID2 = """
admin-0                                                               /admin                                                                                                                                                 kube-service-catalog/default
admin-1                                                               /admin                                                                                                                                                 openshift-ansible-service-broker/asb
asb-access                                                            /asb-access                                                                                                                                            openshift-ansible-service-broker/asb-client
asb-auth                                                              /asb-auth                                                                                                                                              openshift-ansible-service-broker/asb
auth-delegator-openshift-template-service-broker                      /system:auth-delegator                                                                                                                                 openshift-template-service-broker/apiserver
basic-users                                                           /basic-user                                                                                             system:authenticated
cluster-admin                                                         /cluster-admin                                                                                          system:masters
cluster-admin-0                                                       /cluster-admin                                                                                                                                         insights-scan/insights-scan
cluster-admins                                                        /cluster-admin
""".strip()

OC_GET_CLUSTER_ROLE_INVALID = """
name
admin
""".strip()


def test_oc_get_cluster_role_with_config():
    result = openshift_get_with_config.OcGetClusterRoleWithConfig(context_wrap(OC_GET_CLUSTER_ROLE))
    assert result.data[0] == "admin"
    assert "admin" in result
    assert result[0] == "admin"


def test_oc_get_cluster_role_with_config_2():
    with pytest.raises(ParseException) as e:
        openshift_get_with_config.OcGetClusterRoleWithConfig(context_wrap(OC_GET_CLUSTER_ROLE_INVALID))
    assert "invalid" in str(e)


def test_oc_get_clusterrolebinding_with_config():
    result = openshift_get_with_config.OcGetClusterRoleBindingWithConfig(context_wrap(OC_GET_CLUSTERROLEBINDING))
    assert result.rolebinding["admin"] == "/admin"
    assert result.data == [{'NAME': 'admin', 'ROLE': '/admin', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'openshift-infra/template-instance-controller', 'SUBJECTS': ''},
                           {'NAME': 'admin-0', 'ROLE': '/admin', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'kube-service-catalog/default', 'SUBJECTS': ''},
                           {'NAME': 'admin-1', 'ROLE': '/admin', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'openshift-ansible-service-broker/asb', 'SUBJECTS': ''},
                           {'NAME': 'asb-access', 'ROLE': '/asb-access', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'openshift-ansible-service-broker/asb-client', 'SUBJECTS': ''},
                           {'NAME': 'asb-auth', 'ROLE': '/asb-auth', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'openshift-ansible-service-broker/asb', 'SUBJECTS': ''},
                           {'NAME': 'auth-delegator-openshift-template-service-broker',
                            'ROLE': '/system:auth-delegator', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'openshift-template-service-broker/apiserver', 'SUBJECTS': ''},
                           {'NAME': 'basic-users', 'ROLE': '/basic-user', 'USERS': '', 'GROUPS': 'system:authenticated',
                            'SERVICE_ACCOUNTS': '', 'SUBJECTS': ''},
                           {'NAME': 'cluster-admin', 'ROLE': '/cluster-admin', 'USERS': '', 'GROUPS': 'system:masters',
                            'SERVICE_ACCOUNTS': '', 'SUBJECTS': ''},
                           {'NAME': 'cluster-admin-0', 'ROLE': '/cluster-admin', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': 'insights-scan/insights-scan', 'SUBJECTS': ''},
                           {'NAME': 'cluster-admins', 'ROLE': '/cluster-admin', 'USERS': '', 'GROUPS': '',
                            'SERVICE_ACCOUNTS': '', 'SUBJECTS': ''}]


def test_oc_get_clusterrolebinding_with_config_3():
    with pytest.raises(ParseException) as e:
        openshift_get_with_config.OcGetClusterRoleBindingWithConfig(context_wrap(OC_GET_CLUSTERROLEBINDING_INVALID2))
    assert "invalid content" in str(e)


def test_doc():
    env = {
        'oc_get_cluster_role_with_config': OcGetClusterRoleWithConfig(
            context_wrap(OC_GET_CLUSTER_ROLE)),
        'oc_get_clusterrolebinding_with_config': OcGetClusterRoleBindingWithConfig(
            context_wrap(OC_GET_CLUSTERROLEBINDING)),
    }
    failed, total = doctest.testmod(openshift_get_with_config, globs=env)
    assert failed == 0



import pytest
from mock.mock import Mock

from insights.specs.datasources import ipcs
from insights.core.dr import SkipComponent
from insights.specs import Specs


IPCS_OUTPUT1 = """

------ Semaphore Arrays --------
key        semid      owner      perms      nsems
0x00000000 65570      apache     600        1
0x00000000 98353      apache     600        1
0x00000000 98354      apache     600        1
0x00000000 98355      apache     600        1
0x00000000 98356      apache     600        1
0x00000000 98357      apache     600        1
""".strip()

IPCS_OUTPUT2 = """

------ Semaphore Arrays --------
key        semid      owner      perms      nsems
""".strip()


def test_semid():
    ipcs_command = Mock()
    ipcs_command.content = IPCS_OUTPUT1.splitlines()
    broker = {Specs.ipcs_s: ipcs_command}
    result = ipcs.semid(broker)
    assert result is not None
    assert isinstance(result, list)
    assert '65570' in result
    assert '98357' in result
    assert len(result) == 6


def test_exception():
    ipcs_command = Mock()
    ipcs_command.content = IPCS_OUTPUT2.splitlines()
    broker = {Specs.ipcs_s: ipcs_command}
    with pytest.raises(SkipComponent):
        ipcs.semid(broker)



"""
MongodbConf - files - Configuration files for MongoDB
=====================================================

This module contains the following files:
    ``/etc/mongod.conf``,
    ``/etc/mongodb.conf`` ,
    ``/etc/opt/rh/rh-mongodb26/mongod.conf``
    ``/etc/opt/rh/rh-mongodb34/mongod.conf``

They are provided by package mongodb-server, rh-mongodb26-mongodb-server or
rh-mongodb34-mongodb-server.
These MongoDB configuration files may use the **YAML** format
or the standard **key-value pair** format.


Sample input(YAML format)::

    systemLog:
      destination: file
      logAppend: true
      path: /var/log/mongodb/mongod.log

    # Where and how to store data.
    storage:
      dbPath: /var/lib/mongo
      journal:
        enabled: true


Sample input(key-value pair format)::

    # mongodb.conf - generated from Puppet
    #where to log
    logpath=/var/log/mongodb/mongodb.log
    logappend=true
    # Set this option to configure the mongod or mongos process to bind to and
    # listen for connections from applications on this address.
    # You may concatenate a list of comma separated values to bind mongod to multiple IP addresses.
    bind_ip = 127.0.0.1
    # fork and run in background
    fork=true
    dbpath=/var/lib/mongodb
    # location of pidfile
    pidfilepath=/var/run/mongodb/mongodb.pid
    # Enables journaling
    journal = true
    # Turn on/off security.  Off is currently the default
    noauth=true


Examples:

    >>> mongod_conf1 = shared[MongodConf]
    >>> mongod_conf2 = shared[MongodConf]
    >>> MongodbConf1.is_yaml
    True
    >>> MongodbConf2.is_yaml
    False
    >>> mongod_conf1.fork
    True
    >>> mongod_conf2.fork
    'true'
    >>> mongod_conf1.dbpath
    '/var/lib/mongo'
    >>> mongod_conf2.dbpath
    '/var/lib/mongo'
    >>> mongod_conf1.get("systemlog", {}).get("logAppend")
    True
    >>> MongodbConf2.get("logappend")
    'true'

"""

import yaml
from .. import parser, Parser, LegacyItemAccess, get_active_lines
from ..parsers import ParseException, split_kv_pairs
from ..specs import Specs


@parser(Specs.mongod_conf)
class MongodbConf(Parser, LegacyItemAccess):
    """
    Parse the ``/etc/mongod.conf`` config file in key-value pair or YAML format.
    Make several frequently used config options as properties.

    Raises:
        ParseException: Raised when any problem parsing the file content.

    Attributes:
        is_yaml (boolean): True if this is a yaml format file.
    """
    def parse_content(self, content):
        a_content = get_active_lines(content)
        if not a_content:
            raise ParseException("mongod.conf is empty or all lines are comments")
        self.is_yaml = self._file_type_is_yaml(a_content)
        try:
            if self.is_yaml:
                self.data = yaml.safe_load('\n'.join(content))
            else:
                self.data = split_kv_pairs(content, use_partition=True)
        except Exception as e:
            raise ParseException('mongod conf parse failed: %s', e)

    def _file_type_is_yaml(self, content):
        """
        Return True if the file type is YAML.
        Return False means this file will be handled in key-value pair format.

        Why 0.9?
            The normal key-value pair format file would always has the '='
            in each line. Use 0.9 rather than 1 here, just in case there're
            any unexpected lines with wrong settings.
        """

        cnt = sum([1 for line in content if "=" in line])
        percent = float(cnt) / len(content)
        return True if percent < 0.9 else False

    @property
    def bindip(self):
        """
        Return option value of `net.bindIp` if a yaml conf and `bind_ip` if a
        key-value pair conf.
        """
        if self.is_yaml:
            return self.get('net', {}).get('bindIp')
        else:
            return self.get('bind_ip')

    @property
    def port(self):
        """
        Return option value of `net.port` if a yaml conf and `port` if a
        key-value pair conf.
        """
        if self.is_yaml:
            return self.get('net', {}).get('port')
        else:
            return self.get('port')

    @property
    def dbpath(self):
        """
        Return option value of `storage.dbPath` if a yaml conf and `dbPath`
        if a key-value pair conf.
        """
        if self.is_yaml:
            return self.get('storage', {}).get('dbPath') or self.get('storage.dbPath')
        else:
            return self.get('dbpath')

    @property
    def fork(self):
        """
        Return option value of `processManagement.fork` if a yaml conf and
        `fork` if a key-value pair conf.
        """
        if self.is_yaml:
            return self.get('processManagement', {}).get('fork')
        else:
            return self.get('fork')

    @property
    def pidfilepath(self):
        """
        Return option value of `processManagement.pidFilePath` if a yaml conf
        and `pidFilePath` if a key-value pair conf.
        """
        if self.is_yaml:
            return self.get('processManagement', {}).get('pidFilePath')
        else:
            return self.get('pidfilepath')

    @property
    def syslog(self):
        """
        Return option value of `systemLog.destination` if a yaml conf, this
        can be 'file' or 'syslog'. Return value of `syslog` if a key-value pair
        conf, 'true' means log to syslog.
        Return None means value is not specified in configuration file.
        """
        if self.is_yaml:
            return self.get('systemLog', {}).get('destination')
        else:
            return self.get('syslog')

    @property
    def logpath(self):
        """
        Return option value of `systemLog.path` if a yaml conf and `logpath`
        if a key-value pair conf.
        """
        if self.is_yaml:
            return self.get('systemLog', {}).get('path')
        else:
            return self.get('logpath')



"""
AvcHashStats - File ``/sys/fs/selinux/avc/hash_stats``
======================================================

This parser reads the content of ``/sys/fs/selinux/avc/hash_stats``.
"""

from .. import parser, CommandParser, LegacyItemAccess

from ..parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.avc_hash_stats)
class AvcHashStats(CommandParser, LegacyItemAccess):
    """
    Class ``AvcHashStats`` parses the content of the ``/sys/fs/selinux/avc/hash_stats``.

    Attributes:
        entries (int): It is used to show the count of avc hash entries.
        buckets (int): It is used to show the total count of buckets.
        buckets_used (int): It is used to show the count of used buckets.
        longest_chain (int): It is used to show the longest chain.

    A typical sample of the content of this file looks like::

        entries: 509
        buckets used: 290/512
        longest chain: 7


    Examples:
        >>> type(avc_hash_stats)
        <class 'insights.parsers.avc_hash_stats.AvcHashStats'>
        >>> avc_hash_stats.entries
        509
        >>> avc_hash_stats.buckets
        512
        >>> avc_hash_stats.buckets_used
        290
        >>> avc_hash_stats.longest_chain
        7
    """

    def parse_content(self, content):
        self.data = {}
        for line in get_active_lines(content):
            key, value = map(lambda x: x.strip(), line.split(':'))
            self.data.update({key: value})
        self.entries = int(self.data['entries']) if 'entries' in self.data else None
        self.buckets_used, self.buckets = map(lambda x: int(x.strip()),
                                              self.data['buckets used'].split('/')) \
            if 'buckets used' in self.data else [None, None]
        self.longest_chain = int(self.data['longest chain']) if 'longest chain' in self.data else None



"""
LsEdacMC - command ``ls -lan /sys/devices/system/edac/mc``
==========================================================

The ``ls -lan /sys/devices/system/edac/mc`` command provides information for the listing of the
``/sys/devices/system/edac/mc`` directory. See the ``FileListing`` class for a more complete description of the
available features of the class.

Sample ``ls -lan /sys/devices/system/edac/mc`` output::

    /sys/devices/system/edac/mc:
    total 90
    drwxr-xr-x. 3 0 0 0 Jan 10 10:33 .
    drwxr-xr-x. 3 0 0 0 Jan 10 10:33 ..
    drwxr-xr-x. 2 0 0 0 Jan 10 10:33 power
    drwxr-xr-x. 2 0 0 0 Jan 10 10:33 mc0
    drwxr-xr-x. 2 0 0 0 Jan 10 10:33 mc1
    drwxr-xr-x. 2 0 0 0 Jan 10 10:33 mc2

Examples:
    >>> '/sys/devices/system/edac/mc' in ls_edac_mc
    True
    >>> ls_edac_mc.dirs_of('/sys/devices/system/edac/mc') == ['.', '..', 'power', 'mc0', 'mc1', 'mc2']
    True
"""

from insights.specs import Specs
from .. import parser, CommandParser, FileListing


@parser(Specs.ls_edac_mc)
class LsEdacMC(CommandParser, FileListing):
    """
    Parse the /sys/devices/system/edac/mc directory listing using a standard FileListing parser.
    """
    pass



"""
Hostname
========

Combiner for ``hostname`` information. It uses the results of all the
``Hostname`` parsers, ``Facter`` and the ``SystemID`` parser to get the fqdn,
hostname and domain information.


"""

from insights.core.plugins import combiner
from insights.core.serde import deserializer, serializer
from insights.parsers.hostname import Hostname as HnF, HostnameShort as HnS, HostnameDefault as HnD
from insights.parsers.facter import Facter
from insights.parsers.systemid import SystemID
from insights.util import deprecated


@combiner([HnF, HnD, HnS, Facter, SystemID])
class Hostname(object):
    """
    Check hostname, facter and systemid to get the fqdn, hostname and domain.

    Prefer hostname to facter and systemid.

    Examples:
        >>> type(hostname)
        <class 'insights.combiners.hostname.Hostname'>
        >>> hostname.fqdn
        'rhel7.example.com'
        >>> hostname.hostname
        'rhel7'
        >>> hostname.domain
        'example.com'

    Raises:
        Exception: If no hostname can be found in any of the source parsers.
    """
    def __init__(self, hf, hd, hs, ft, sid):
        self.fqdn = self.hostname = self.domain = None

        if hf or hs or hd or ft:
            hn = hf or hs or hd or ft
            self.hostname = self.fqdn = hn.hostname
            self.domain = ''
            if hf and hf.fqdn:
                self.fqdn = hf.fqdn
                self.domain = hf.domain
            elif ft and ft.fqdn:
                self.fqdn = ft.fqdn
                self.domain = ft.domain if ft.domain else ".".join(self.fqdn.split(".")[1:])
        else:
            self.fqdn = sid.get("profile_name")
            if self.fqdn:
                self.hostname = self.fqdn.split(".")[0]
                self.domain = ".".join(self.fqdn.split(".")[1:])

        if not self.hostname or not self.fqdn:
            raise Exception("Unable to get hostname.")


@combiner([HnF, HnD, HnS, Facter, SystemID])
def hostname(hf, hd, hs, ft, sid):
    """
    .. warning::
        This combiner methode is deprecated, please use
        :py:class:`insights.combiners.hostname.Hostname` instead.

    Check hostname, facter and systemid to get the fqdn, hostname and domain.

    Prefer hostname to facter and systemid.

    Examples:
        >>> hn.fqdn
        'rhel7.example.com'
        >>> hn.hostname
        'rhel7'
        >>> hn.domain
        'example.com'

    Returns:
        insights.combiners.hostname.Hostname: A class with `fqdn`,
        `hostname` and `domain` attributes.

    Raises:
        Exception: If no hostname can be found in any of the source parsers.
    """
    deprecated(hostname, "Use the `Hostname` class instead.")
    return Hostname(hf, hd, hs, ft, sid)


@serializer(Hostname)
def serialize(obj, root=None):
    return {"fqdn": obj.fqdn, "hostname": obj.hostname, "domain": obj.domain}


@deserializer(Hostname)
def deserialize(_type, data, root=None):
    return Hostname(**data)



from insights.parsers import SkipException, subscription_manager_release
from insights.parsers.subscription_manager_release import SubscriptionManagerReleaseShow
from insights.tests import context_wrap
import pytest
import doctest

INPUT_NORMAL_1 = """
Release: 7.2
""".strip()

INPUT_NORMAL_2 = """
Release: 6Server
""".strip()

INPUT_NORMAL_3 = """
Release: 8
""".strip()

INPUT_NOT_SET = """
Release not set
""".strip()

INPUT_NG_1 = """
XYC
Release not set
""".strip()

INPUT_NG_2 = """
Release: 7.x
""".strip()

INPUT_NG_3 = """
Release: 7.x DUMMY
""".strip()

INPUT_NG_4 = """
Release: 7x
""".strip()


def test_subscription_manager_release_show_ok():
    ret = SubscriptionManagerReleaseShow(context_wrap(INPUT_NORMAL_1))
    assert ret.set == '7.2'
    assert ret.major == 7
    assert ret.minor == 2

    ret = SubscriptionManagerReleaseShow(context_wrap(INPUT_NORMAL_2))
    assert ret.set == '6Server'
    assert ret.major == 6
    assert ret.minor is None

    ret = SubscriptionManagerReleaseShow(context_wrap(INPUT_NORMAL_3))
    assert ret.set == '8'
    assert ret.major == 8
    assert ret.minor is None


def test_subscription_manager_release_show_not_set():
    ret = SubscriptionManagerReleaseShow(context_wrap(INPUT_NOT_SET))
    assert ret.set is None
    assert ret.major is None
    assert ret.minor is None


def test_subscription_manager_release_show_ng():
    with pytest.raises(SkipException) as e_info:
        SubscriptionManagerReleaseShow(context_wrap(INPUT_NG_1))
    assert "Content takes at most 1 line (2 given)." in str(e_info.value)

    with pytest.raises(SkipException) as e_info:
        SubscriptionManagerReleaseShow(context_wrap(INPUT_NG_2))
    assert "Incorrect content: Release: 7.x" in str(e_info.value)

    with pytest.raises(SkipException) as e_info:
        SubscriptionManagerReleaseShow(context_wrap(INPUT_NG_3))
    assert "Incorrect content: Release: 7.x DUMMY" in str(e_info.value)

    with pytest.raises(SkipException) as e_info:
        SubscriptionManagerReleaseShow(context_wrap(INPUT_NG_4))
    assert "Incorrect content: Release: 7x" in str(e_info.value)


def test_doc_examples():
    env = {
            'rhsm_rel':
            SubscriptionManagerReleaseShow(context_wrap(INPUT_NORMAL_1)),
          }
    failed, total = doctest.testmod(subscription_manager_release, globs=env)
    assert failed == 0



import os
import sys
from insights import run, make_fail, make_pass
from insights.core import dr
from insights.plugins import always_fires, never_fires
from insights.specs import Specs
from mock import patch
from insights.tests.spec_tests import report_multioutput, report_nonexistent, report_raw


class stage(dr.ComponentType):
    pass


@stage("dep1")
def stage1(dep1):
    return "stage1"


@stage("dep2")
def stage2(dep2):
    return "stage2"


@stage("common")
def stage3(common):
    return common


@stage("common")
def stage4(common):
    return common


def test_run():
    broker = dr.Broker()
    broker["common"] = 3
    graph = dr.get_dependency_graph(stage3)
    graph.update(dr.get_dependency_graph(stage4))
    broker = dr.run(graph, broker)

    assert stage3 in broker.instances
    assert broker[stage3] == 3

    assert stage4 in broker.instances
    assert broker[stage4] == 3


def test_run_incremental():
    broker = dr.Broker()
    broker["dep1"] = 1
    broker["dep2"] = 2
    broker["common"] = 3

    graph = dr.get_dependency_graph(stage1)
    graph.update(dr.get_dependency_graph(stage2))
    graph.update(dr.get_dependency_graph(stage3))
    graph.update(dr.get_dependency_graph(stage4))

    brokers = list(dr.run_incremental(graph, broker))
    assert len(brokers) == 3


ALWAYS_FIRES_RESULT = make_pass("ALWAYS_FIRES", kernel="this is junk")
NEVER_FIRES_RESULT = {
    'rule_fqdn': 'insights.plugins.never_fires.report',
    'reason': 'MISSING_REQUIREMENTS',
    'details': "All: ['insights.plugins.never_fires.thing'] Any: ",
    'type': 'skip'}
REDHAT_RELEASE = "Red Hat Enterprise Linux Server release 7.3 (Maipo)"
UNAME = "Linux test.redhat.com 3.10.0-514.el7.x86_64 #1 SMP Wed Oct 19 11:24:13 EDT 2016 x86_64 x86_64 x86_64 GNU/Linux"


def test_run_command(tmpdir):

    d = tmpdir / "etc"
    d.mkdir()
    p = d / "redhat-release"
    p.write(REDHAT_RELEASE)
    g = tmpdir / "insights_commands"
    g.mkdir()
    r = g / "uname_-a"
    r.write(UNAME)

    broker = run([always_fires.report, never_fires.report])
    assert broker is not None
    assert always_fires.report in broker
    assert never_fires.report in broker
    assert broker[always_fires.report] == ALWAYS_FIRES_RESULT
    assert broker[never_fires.report] == NEVER_FIRES_RESULT

    broker = run([Specs.redhat_release, always_fires.report, never_fires.report], root=tmpdir.strpath)
    assert broker is not None
    assert always_fires.report in broker
    assert never_fires.report in broker
    assert broker[always_fires.report] == ALWAYS_FIRES_RESULT
    assert broker[never_fires.report] == NEVER_FIRES_RESULT
    assert Specs.redhat_release in broker
    assert broker[Specs.redhat_release].content == [REDHAT_RELEASE]

    testargs = ["insights-run", "-p", "insights.plugins"]
    with patch.object(sys, 'argv', testargs):
        broker = run(print_summary=True)
        assert broker is not None
        assert always_fires.report in broker
        assert never_fires.report in broker
        assert broker[always_fires.report] == ALWAYS_FIRES_RESULT
        assert broker[never_fires.report] == NEVER_FIRES_RESULT

    testargs = ["insights-run", "-p", "insights.plugins", tmpdir.strpath]
    with patch.object(sys, 'argv', testargs):
        broker = run(print_summary=True)
        assert broker is not None
        assert always_fires.report in broker
        assert never_fires.report in broker
        assert broker[always_fires.report] == ALWAYS_FIRES_RESULT
        assert broker[never_fires.report] == NEVER_FIRES_RESULT
        assert Specs.uname in broker
        assert broker[Specs.uname].content == [UNAME]


SAMPLE_LOG = """
1 line one
2 line two
""".strip()


def test_bare_files(tmpdir):
    d = tmpdir / "bare"
    d.mkdir()
    p = d / 'sample.log'
    p.write(SAMPLE_LOG)
    data = [l.strip() for l in SAMPLE_LOG.splitlines()]

    testargs = [
        "insights-run",
        "-t", "-m",
        "-b", 'insights.tests.spec_tests.TSpecs.sample_multioutput_file={fname}'.format(fname=os.path.join(tmpdir.strpath, 'bare', 'sample.log')),
        "-p", "insights.tests.spec_tests"]
    with patch.object(sys, 'argv', testargs):
        broker = run(print_summary=True)
        assert broker is not None
        assert report_multioutput in broker
        assert broker[report_multioutput] == make_fail('MO_SPEC', data=data, number=1)

    testargs = [
        "insights-run",
        "-t", "-m",
        "-b", 'insights.tests.spec_tests.TSpecs.sample_nonexistent={fname}'.format(fname=os.path.join(tmpdir.strpath, 'bare', 'sample.log')),
        "-p", "insights.tests.spec_tests"]
    with patch.object(sys, 'argv', testargs):
        broker = run(print_summary=True)
        assert broker is not None
        assert report_nonexistent in broker
        assert broker[report_nonexistent] == make_fail('NE_SPEC', data=data)

    testargs = [
        "insights-run",
        "-t", "-m",
        "-b", 'insights.tests.spec_tests.TSpecs.sample_raw_file={fname}'.format(fname=os.path.join(tmpdir.strpath, 'bare', 'sample.log')),
        "-p", "insights.tests.spec_tests"]
    with patch.object(sys, 'argv', testargs):
        broker = run(print_summary=True)
        assert broker is not None
        assert report_raw in broker
        with open(os.path.join(tmpdir.strpath, 'bare', 'sample.log'), 'rb') as fh:
            data_b = fh.read()
        assert broker[report_raw] == make_fail('RA_SPEC', data=data_b)



"""
Units Manged By Systemctl (services)
====================================

Parsers included in this module are:

ListUnits - command ``/bin/systemctl list-units``
-------------------------------------------------

UnitFiles - command ``/bin/systemctl list-unit-files``
------------------------------------------------------

"""
from .. import get_active_lines
from ... import Parser, parser
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.systemctl_list_unit_files)
class UnitFiles(Parser):
    """
    The UnitFiles class parses the output of ``/bin/systemctl list-unit-files`` and provides
    information about enabled services.

    Output of Command::
        UNIT FILE                                     STATE
        mariadb.service                               enabled
        neutron-openvswitch-agent.service             enabled
        neutron-ovs-cleanup.service                   enabled
        neutron-server.service                        enabled
        runlevel0.target                              disabled
        runlevel1.target                              disabled
        runlevel2.target                              enabled

    Raises:
        SkipException: When nothing is parsed.

    Example:

        >>> conf.is_on('mariadb.service')
        True
        >>> conf.is_on('runlevel0.target')
        False
        >>> conf.exists('neutron-server.service')
        True
        >>> conf.exists('runlevel1.target')
        True
        >>> 'mariadb.service' in conf.services
        True
        >>> 'runlevel0.target' in conf.services
        True
        >>> 'nonexistent-service.service' in conf.services
        False
        >>> conf.services['mariadb.service']
        True
        >>> conf.services['runlevel1.target']
        False
        >>> conf.services['nonexistent-service.service']
        Traceback (most recent call last):
          File "<doctest insights.parsers.systemd.unitfiles.UnitFiles[11]>", line 1, in <module>
            conf.services['nonexistent-service.service']
        KeyError: 'nonexistent-service.service'
    """

    def __init__(self, *args, **kwargs):
        self.services = {}
        """dict: Dictionary of bool indicating if service is enabled,
        access by service name ."""
        self.service_list = []
        """list: List of service names in order of appearance."""
        self.parsed_lines = {}
        """dict: Dictionary of content lines access by service name."""
        super(UnitFiles, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        """
        Main parsing class method which stores all interesting data from the content.

        Args:
            content (context.content): Parser context content
        """
        # 'static' means 'on' to fulfill dependency of something else that is on
        # man systemctl - "is-enabled" knows these states
        valid_states = set(['enabled', 'enabled-runtime', 'linked', 'linked-runtime', 'masked',
                            'masked-runtime', 'static', 'indirect', 'disabled', 'generated',
                            'transient', 'bad', 'invalid'])
        # man systemctl - "is-enabled" considers these to be enabled
        on_states = set(['enabled', 'enabled-runtime', 'static', 'indirect', 'generated', 'transient'])

        for line in get_active_lines(content):
            parts = line.split(None)  # AWK like split, strips whitespaces
            if len(parts) == 2 and any(part in valid_states for part in parts):
                service, state = parts
                enabled = state in on_states
                self.services[service] = enabled
                self.parsed_lines[service] = line
                self.service_list.append(service)

        if not self.services:
            raise SkipException

    def is_on(self, service_name):
        """
        Checks if the service is enabled in systemctl.

        Args:
            service_name (str): service name including '.service'

        Returns:
            Union[bool, None]: True if service is enabled, False if it is disabled. None if the
            service doesn't exist.
        """
        return self.services.get(service_name, None)

    def exists(self, service_name):
        """
        Checks if the service is listed in systemctl.

        Args:
           service_name (str): service name including '.service'

        Returns:
            bool: True if service exists, False otherwise.
        """
        return service_name in self.service_list


@parser(Specs.systemctl_list_units)
class ListUnits(Parser):

    """
    The ListUnits class parses the output of ``/bin/systemctl list-units`` and provides
    information about all the services listed under it.

    Output of Command::

        UNIT                                LOAD   ACTIVE SUB       DESCRIPTION
        sockets.target                      loaded active active    Sockets
        swap.target                         loaded active active    Swap
        systemd-shutdownd.socket            loaded active listening Delayed Shutdown Socket
        neutron-dhcp-agent.service          loaded active running   OpenStack Neutron DHCP Agent
        neutron-openvswitch-agent.service   loaded active running   OpenStack Neutron Open vSwitch Agent
        ...
        unbound-anchor.timer                loaded active waiting   daily update of the root trust anchor for DNSSEC

        LOAD   = Reflects whether the unit definition was properly loaded.
        ACTIVE = The high-level unit activation state, i.e. generalization of SUB.
        SUB    = The low-level unit activation state, values depend on unit type.

        161 loaded units listed. Pass --all to see loaded but inactive units, too.
        To show all installed unit files use 'systemctl list-unit-files'.

    Raises:
        SkipException: When nothing is parsed.

    Example:

        >>> units.get_service_details('swap.target') == {'LOAD': 'loaded', 'ACTIVE': 'active', 'SUB': 'active', 'UNIT': 'swap.target', 'DESCRIPTION': 'Swap'}
        True
        >>> units.unit_list['swap.target'] == {'LOAD': 'loaded', 'ACTIVE': 'active', 'SUB': 'active', 'UNIT': 'swap.target', 'DESCRIPTION': 'Swap'}
        True
        >>> units.is_active('swap.target')
        True
        >>> units.get_service_details('random.service') == {'LOAD': None, 'ACTIVE': None, 'SUB': None, 'UNIT': None, 'DESCRIPTION': None}
        True
    """
    EMPTY_DETAILS = {'LOAD': None, 'ACTIVE': None, 'SUB': None, 'UNIT': None, 'DESCRIPTION': None}

    def __init__(self, *args, **kwargs):
        self.unit_list = {}
        """dict: Dictionary service detail like active, running, exited, dead"""
        super(ListUnits, self).__init__(*args, **kwargs)

    def parse_service_details(self, parts):
        # man systemctl - "is-enabled" knows these states
        valid_states = set(['invalid', 'loaded', 'inactive', 'active',
                            'exited', 'running', 'failed', 'mounted', 'waiting', 'plugged'])

        valid_units = set(['service', 'socket', 'device', 'mount', 'automount', 'swap', 'target',
                           'path', 'timer', 'slice', 'scope'])

        service_details = {}
        if (len(parts) >= 4 and any(part in valid_states for part in parts) and
                any(unit in parts[0] for unit in valid_units)):
            service_details['UNIT'] = parts[0]
            service_details['LOAD'] = parts[1]
            service_details['ACTIVE'] = parts[2]
            service_details['SUB'] = parts[3]
            service_details['DESCRIPTION'] = ' '.join(parts[4:]) if len(parts) > 4 else None

        return service_details

    def parse_content(self, content):
        """
        Main parsing class method which stores all interesting data from the content.

        Args:
            content (context.content): Parser context content
        """
        BULLET_CHAR_U = u'\u25CF'
        BULLET_CHAR_B = b"\xe2\x97\x8f"
        for line in get_active_lines(content):
            # If this is a heading line, then ignore this line
            if line.startswith('UNIT '):
                continue

            parts = line.split(None)  # AWK like split, strips whitespaces
            first_part = 0
            if parts[0] == BULLET_CHAR_U or parts[0].encode('utf-8') == BULLET_CHAR_B or parts[0] == '*':
                first_part = 1

            # If past the end of the list then quit
            if parts[first_part] in ['LOAD', 'ACTIVE', 'SUB']:
                break

            service_details = self.parse_service_details(parts[first_part:])
            if service_details:
                self.unit_list[parts[first_part]] = service_details

        if not self.unit_list:
            raise SkipException

    def get_service_details(self, service_name):
        """
        Return the service details collected by systemctl.

        Args:
            service_name (str): service name including its extension.

        Returns:
            dict: Dictionary containing details for the service.
            if service is not present dictonary values will be `None`::

            {'LOAD': 'loaded', 'ACTIVE': 'active', 'SUB': 'running', 'UNIT': 'neutron-dhcp-agent.service'}
        """
        return self.unit_list.get(service_name, ListUnits.EMPTY_DETAILS)

    def is_loaded(self, service_name):
        """
        Return the LOAD state of service managed by systemd.

        Args:
            service_name (str): service name including its extension.

        Returns:
            bool: True if service is loaded False if not loaded
        """
        return self.get_service_details(service_name)['LOAD'] == 'loaded'

    def is_active(self, service_name):
        """
        Return the ACTIVE state of service managed by systemd.

        Args:
            service_name (str): service name including its extension.

        Returns:
            bool: True if service is active False if inactive
        """
        return self.get_service_details(service_name)['ACTIVE'] == 'active'

    def is_running(self, service_name):
        """
        Return the SUB state of service managed by systemd.

        Args:
            service_name (str): service name including its extension.

        Returns:
            bool: True if service is running False in all other states.
        """
        return self.get_service_details(service_name)['SUB'] == 'running'

    def is_failed(self, service_name):
        """
        Return the ACTIVE state of service managed by systemd.

        Args:
            service_name (str): service name including its extension.

        Returns:
            bool: True if service is failed, False in all other states.
        """
        return self.get_service_details(service_name)['ACTIVE'] == 'failed'

    @property
    def service_names(self):
        """list: Returns a list of all UNIT names."""
        return list(self.unit_list.keys())



from insights.parsers.neutron_conf import NeutronConf
from insights.tests import context_wrap

NEUTRON_CONF = """
[DEFAULT]
# debug = False
debug = False
# verbose = True
verbose = False
core_plugin =neutron.plugins.openvswitch.ovs_neutron_plugin.OVSNeutronPluginV2

[quotas]
default_quota = -1
quota_network = 10
[agent]
report_interval = 60

[keystone_authtoken]
auth_host = ost-controller-lb-del.om-l.dsn.inet
auth_port = 35357
[database]
connection = mysql://neutron:dSNneutron01@ost-mysql.om-l.dsn.inet/neutron?ssl_ca=/etc/pki/CA/certs/ca.crt
[service_providers]
service_provider = LOADBALANCER:Haproxy:neutron.services.loadbalancer.drivers.haproxy.plugin_driver.HaproxyOnHostPluginDriver:default
"""


def test_neutron_conf():
    nconf = NeutronConf(context_wrap(NEUTRON_CONF))
    assert nconf is not None
    assert list(nconf.sections()) == ['quotas', 'agent', 'keystone_authtoken', 'database', 'service_providers']
    assert nconf.defaults() == {
        'debug': 'False',
        'verbose': 'False',
        'core_plugin': 'neutron.plugins.openvswitch.ovs_neutron_plugin.OVSNeutronPluginV2'}
    assert nconf.get('quotas', 'quota_network') == '10'
    assert nconf.has_option('database', 'connection')
    assert not nconf.has_option('yabba', 'dabba_do')
    assert nconf.get('DEFAULT', 'debug') == 'False'



import doctest

from insights.parsers import cron_daily_rhsmd
from insights.parsers.cron_daily_rhsmd import CronDailyRhsmd
from insights.tests import context_wrap

RHSMD_1 = """
config=$(grep -E "^processTimeout" /etc/rhsm/rhsm.conf | grep -Po "[0-9]+")
rhsmd_timeout=$config
abc=$config
""".strip()


def test_docs():
    CronDailyRhsmd.collect('config_lines', lambda n: n if "$config" in n else "")
    CronDailyRhsmd.any('one_config_line', lambda n: n if "$config" in n else "")
    env = {
        'rhsmd': CronDailyRhsmd(context_wrap(RHSMD_1))
    }
    failed, total = doctest.testmod(cron_daily_rhsmd, globs=env)
    assert failed == 0


def test_parser():
    cron_daily_rhsmd.CronDailyRhsmd.collect('config_varialbe_lines', lambda n: n if "$config" in n else "")
    cron_daily_rhsmd.CronDailyRhsmd.any('rhsmd_timeout', lambda n: n if "rhsmd_timeout" in n else "")
    rhs = cron_daily_rhsmd.CronDailyRhsmd(context_wrap(RHSMD_1))
    assert rhs.rhsmd_timeout == 'rhsmd_timeout=$config'
    assert rhs.config_varialbe_lines == ['rhsmd_timeout=$config', 'abc=$config']



"""
Facter - command ``/usr/bin/facter``
====================================

Module for the parsing of output from the ``facter`` command.
Data is avaliable as a dict for each line of the output.

Sample input data for the ``facter`` command looks like::

    architecture => x86_64
    bios_vendor => Phoenix Technologies LTD
    bios_version => 6.00
    domain => example.com
    facterversion => 1.7.6
    filesystems => btrfs,ext2,ext3,ext4,msdos,vfat,xfs
    fqdn => plin-w1rhns01.example.com
    hostname => plin-w1rhns01
    ipaddress => 172.23.27.50
    ipaddress_ens192 => 172.23.27.50
    ipaddress_lo => 127.0.0.1
    is_virtual => true
    kernel => Linux
    kernelmajversion => 3.10

Examples:
    >>> facts_info = shared[facter]
    >>> facts_info.kernelmajversion
    '3.10'
    >>> facts_info.domain
    'example.com'
    >>> facts_info.architecture
    'x86_64'
"""

from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs


@parser(Specs.facter)
class Facter(LegacyItemAccess, CommandParser):
    """Class for parsing ``facter`` command output.

    Attributes are the facts in each line of the command output.
    All facts may be accessed as ``obj.fact_name``. The ``get`` method is also
    provided to access any facts.
    """
    def parse_content(self, content):
        """
        Main parsing class method which stores all interesting data from the content.

        Args:
            content (context.content): Parser context content

        Returns:
            dict: dictionary with parsed data
        """
        self.fqdn = self.hostname = self.domain = None
        facts_info = {}
        for line in content:
            if ' => ' in line:
                key, value = [k.strip() for k in line.split('=>', 1)]
                facts_info[key] = value

        for k, v in facts_info.items():
            setattr(self, k, v)

        self.data = facts_info



"""
LsSCSI - command ``/usr/bin/lsscsi``
====================================

This module provides processing for the output of the ``/usr/bin/lsscsi`` command.
"""

from . import ParseException
from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.lsscsi)
class LsSCSI(CommandParser):
    """
    This parser reads the output of ``/usr/bin/lsscsi`` into a list of dictionarys.
    Each item is a dictionary with six keys:

    * ``HCTL`` - the scsi_host,channel,target_number,LUN tuple
    * ``Peripheral-Type`` - the SCSI peripheral type
    * ``Vendor`` - the vendor name
    * ``Model`` - the model name
    * ``Revision`` - the revision string
    * ``Primary-Device-Node`` - the primary device node name

    Attributes:
        data (list of dict): List of the input lines, where each line is a dictionary having the keys identified above.

    Parsing refers to http://sg.danny.cz/scsi/lsscsi.html.

    Sample input::

        [1:0:0:0]    storage IET      Controller       0001  -
        [1:0:0:1]    cd/dvd  QEMU     QEMU DVD-ROM     2.5+  /dev/sr0
        [1:0:0:2]    disk    IET      VIRTUAL-DISK     0001  /dev/sdb
        [3:0:5:0]    tape    HP       C5713A           H910  /dev/st0

    Examples:

        >>> lsscsi[0] == {'Model': 'Controller', 'Vendor': 'IET', 'HCTL': '[1:0:0:0]', 'Peripheral-Type': 'storage', 'Primary-Device-Node': '-', 'Revision': '0001'}
        True
        >>> lsscsi.device_nodes
        ['-', '/dev/sr0', '/dev/sdb', '/dev/st0']
        >>> len(lsscsi.data)
        4
        >>> lsscsi[1]['Peripheral-Type']
        'cd/dvd'

    """
    def parse_content(self, content):
        if len(content) == 0:
            raise ParseException("Empty content of command output.")

        LSSCSI_TABLE_HEADER_ITEMS = ['HCTL', 'Peripheral-Type', 'Vendor', 'Model', 'Revision', 'Primary-Device-Node']
        LEN = len(LSSCSI_TABLE_HEADER_ITEMS)

        col_index = []
        pre_col_index = []
        # Try to find the index of a proper six column line.
        # If col_index can't be found in the above proper way,
        # set it by squeezing redundant items to 'Model' column as a workaround.
        for l in content:
            col_split = l.strip().split()
            if len(col_split) < LEN:
                break
            elif len(col_split) == LEN:
                col_index = [l.index(c) for c in col_split]
                break
            elif not pre_col_index and len(col_split) > LEN:
                unfixed_index = [l.index(c) for c in col_split]
                forth_str = l.split(col_split[2], 1)[-1].strip()
                pre_col_index = unfixed_index[:3] + [l.index(forth_str)] + unfixed_index[-2:]

        if not col_index and pre_col_index:
            col_index = pre_col_index

        if len(col_index) != LEN:
            raise ParseException("Invalid format of content, unparsable.")

        self.data = []
        for line in content:
            col_data = dict((LSSCSI_TABLE_HEADER_ITEMS[i],
                             line[col_index[i]:col_index[i + 1]].strip())
                            for i in range(LEN - 1))
            col_data[LSSCSI_TABLE_HEADER_ITEMS[-1]] = line[col_index[-1]:].strip()
            self.data.append(col_data)

    def __getitem__(self, idx):
        return self.data[idx]

    @property
    def device_nodes(self):
        """
        list: All lines' Primary-Device-Node values.
        """
        return [v['Primary-Device-Node'] for v in self.data]

    @property
    def device_vendors(self):
        """
        list: All lines' Vendor values
        """
        return [v['Vendor'] for v in self.data]



""""
test mount
==========
"""
from insights.parsers import ParseException, SkipException
from insights.parsers import mount
from insights.parsers.mount import Mount, ProcMounts
from insights.tests import context_wrap
import pytest
import doctest

MOUNT_DATA = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
nfsd on /proc/fs/nfsd type nfsd (rw,relatime)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-home on /home type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
sunrpc on /var/lib/nfs/rpc_pipefs type rpc_pipefs (rw,relatime)
tmpfs on /run/user/42 type tmpfs (rw,nosuid,nodev,relatime,seclabel,size=1605428k,mode=700,uid=42,gid=42)
tmpfs on /run/user/1000 type tmpfs (rw,nosuid,nodev,relatime,seclabel,size=1605428k,mode=700,uid=1000,gid=1000)
gvfsd-fuse on /run/user/1000/gvfs type fuse.gvfsd-fuse (rw,nosuid,nodev,relatime,user_id=1000,group_id=1000)
fusectl on /sys/fs/fuse/connections type fusectl (rw,relatime)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered) [CONFIG]
/dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
""".strip()

# Missing 'on' in second line
MOUNT_ERR_DATA = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
""".strip()

MOUNT_WITHOUT_ROOT = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
""".strip()


PROC_MOUNT = """
proc /proc proc rw,relatime 0 0
sysfs /sys sysfs rw,relatime 0 0
devtmpfs /dev devtmpfs rw,relatime,size=8155456k,nr_inodes=2038864,mode=755 0 0
devpts /dev/pts devpts rw,relatime,gid=5,mode=620,ptmxmode=000 0 0
tmpfs /dev/shm tmpfs rw,nosuid,nodev,noexec,relatime 0 0
/dev/mapper/rootvg-rootlv / ext4 rw,relatime,barrier=1,data=ordered 0 0
/proc/bus/usb /proc/bus/usb usbfs rw,relatime 0 0
/dev/sda1 /boot ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-homelv /home ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-optlv /opt ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-tmplv /tmp ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-usrlv /usr ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-varlv /var ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-tmplv /var/tmp ext4 rw,relatime,barrier=1,data=ordered 0 0
none /proc/sys/fs/binfmt_misc binfmt_misc rw,relatime 0 0
sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
/vol/vol3/wpsanet /sa/net nfs rw,relatime,vers=3,rsize=8192,wsize=8192,namlen=255,soft,proto=tcp,timeo=14,retrans=2,sec=sys,mountaddr=10.xx.xx.xx,mountvers=3,mountport=4046,mountproto=udp,local_lock=none 0 0
/vol/vol9/home /users nfs rw,relatime,vers=3,rsize=8192,wsize=8192,namlen=255,soft,proto=tcp,timeo=14,retrans=2,sec=sys,mountaddr=10.xx.xx.xx,mountvers=3,mountport=4046,mountproto=udp,local_lock=none,addr=10.xx.xx.xx 0 0
/etc/auto.misc /misc autofs rw,relatime,fd=7,pgrp=1936,timeout=300,minproto=5,maxproto=5,indirect 0 0
""".strip()

PROCMOUNT_ERR_DATA = """
rootfs / rootfs rw 0 0
sysfs /sys sysfs rw,relatime
""".strip()

PROC_EXCEPTION1 = """
""".strip()

PROC_EXCEPTION2 = """
proc /proc proc rw,relatime 0 0
sysfs /sys sysfs rw,relatime 0 0
devtmpfs /dev devtmpfs rw,relatime,size=8155456k,nr_inodes=2038864,mode=755 0 0
""".strip()

MOUNT_DOC = """
/dev/mapper/rootvg-rootlv on / type ext4 (rw,relatime,barrier=1,data=ordered)
proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered)
dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
""".strip()

PROC_MOUNT_DOC = """
/dev/mapper/rootvg-rootlv / ext4 rw,relatime,barrier=1,data=ordered 0 0
proc /proc proc rw,nosuid,nodev,noexec,relatime 0 0
/dev/mapper/HostVG-Config /etc/shadow ext4 rw,noatime,seclabel,stripe=256,data=ordered 0 0
dev/sr0 /run/media/root/VMware\040Tools iso9660 ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2 0 0
""".strip()

MOUNT_DATA_WITH_SPECIAL_MNT_POINT = """
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered) [CONFIG]
/dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
/dev/sr1 on /run/media/Disk on C type NFS type iso9660 (ro,uid=0,gid=0,iocharset=utf8,uhelper=udisks2) [C type Disk]
""".strip()


def test_mount():
    results = Mount(context_wrap(MOUNT_DATA))
    assert results is not None
    assert len(results) == 13
    sr0 = results.search(filesystem='/dev/sr0')[0]
    sda1 = results.search(filesystem='/dev/sda1')[0]
    assert sr0 is not None
    assert sr0['mount_point'] == '/run/media/root/VMware Tools'
    # test get method
    assert sr0.get('mount_point') == '/run/media/root/VMware Tools'
    assert sr0.get('does not exist', 'failure') == 'failure'
    assert sr0['mount_type'] == 'iso9660'
    assert 'ro' in sr0['mount_options']
    assert sr0.mount_options.ro
    assert 'relatime' in sr0['mount_options']
    assert sr0['mount_options']['uhelper'] == 'udisks2'
    assert sr0['mount_label'] == '[VMware Tools]'
    assert sda1 is not None
    assert sda1['mount_point'] == '/boot'
    assert sda1['mount_type'] == 'ext4'
    assert 'rw' in sda1['mount_options']
    assert 'seclabel' in sda1['mount_options']
    assert sda1['mount_options']['data'] == 'ordered'
    assert sda1.mount_options.data == 'ordered'
    assert 'mount_label' not in sda1

    # Test iteration
    for mnt in results:
        assert hasattr(mnt, 'filesystem')
        assert hasattr(mnt, 'mount_point')
        assert hasattr(mnt, 'mount_type')
        assert hasattr(mnt, 'mount_options')

    # Test getitem
    assert results[12] == sr0
    assert results['/etc/shadow'] == results[11]
    # Index only by string or number
    with pytest.raises(TypeError) as exc:
        assert results[set([1, 2, 3])] is None
    assert "Mounts can only be indexed by mount string or line number" in str(exc)

    # Test mounts dictionary
    assert results.mounts['/run/media/root/VMware Tools'] == sr0

    # Test get_dir
    assert results.get_dir('/run/media/root/VMware Tools') == sr0
    assert results.get_dir('/boot/grub2/grub.cfg') == sda1
    assert results.get_dir('/etc') == results['/']
    assert results.get_dir('relative/paths/fail') is None

    # Test search
    assert results.search(filesystem='/dev/sr0') == [sr0]
    assert results.search(mount_type='tmpfs') == [
        results.rows[n] for n in (0, 7, 8)
    ]
    assert results.search(mount_options__contains='seclabel') == [
        results.rows[n] for n in (0, 1, 3, 4, 5, 7, 8, 11)
    ]


def test_mount_with_special_mnt_point():
    results = Mount(context_wrap(MOUNT_DATA_WITH_SPECIAL_MNT_POINT))
    assert len(results) == 6
    sr0 = results.search(filesystem='/dev/sr0')[0]
    sr1 = results.search(filesystem='/dev/sr1')[0]
    assert sr0['mount_point'] == '/run/media/root/VMware Tools'
    assert sr1['mount_point'] == '/run/media/Disk on C type NFS'
    assert sr0.get('mount_label') == '[VMware Tools]'
    assert sr1.get('mount_label') == '[C type Disk]'
    assert results['/run/media/Disk on C type NFS'].get('mount_label') == '[C type Disk]'


def test_mount_exception1():
    # Test parse failure
    with pytest.raises(ParseException) as pe:
        Mount(context_wrap(MOUNT_ERR_DATA))
    assert 'Unable to parse' in str(pe.value)


def test_mount_exception2():
    with pytest.raises(ParseException) as exc:
        Mount(context_wrap(MOUNT_WITHOUT_ROOT))
    assert "Input for mount must contain '/' mount point." in str(exc)


def test_proc_mount():
    results = ProcMounts(context_wrap(PROC_MOUNT))
    assert results is not None
    assert len(results) == 19
    sda1 = results.search(filesystem='/dev/sda1')[0]

    # Test get method
    assert sda1 is not None
    assert sda1['mount_point'] == '/boot'
    assert sda1['mount_type'] == 'ext4'
    assert 'rw' in sda1['mount_options']
    assert 'relatime' in sda1['mount_options']
    assert sda1['mount_options']['data'] == 'ordered'
    assert sda1.mount_options.data == 'ordered'
    assert sda1['mount_label'] == ['0', '0']

    # Test iteration
    for mnt in results:
        assert hasattr(mnt, 'filesystem')
        assert hasattr(mnt, 'mount_point')
        assert hasattr(mnt, 'mount_type')
        assert hasattr(mnt, 'mount_options')

    # Test getitem
    assert results[7] == sda1
    assert results['/misc'] == results[-1]
    # Index only by string or number
    with pytest.raises(TypeError) as exc:
        assert results[set([1, 2, 3])] is None
    assert 'Mounts can only be indexed by mount string or line number' in str(exc)

    # Test mounts dictionary
    assert results.mounts['/boot'] == sda1

    # Test get_dir
    assert results.get_dir('/var/lib/nfs/rpc_pipefs') == results.search(filesystem='sunrpc')[0]
    assert results.get_dir('/etc') == results['/']

    # Test search
    assert results.search(filesystem='/dev/sda1') == [sda1]
    assert results.search(mount_type='nfs') == [
        results.rows[n] for n in (16, 17)
    ]
    assert results.search(mount_options__contains='mode') == [
        results.rows[n] for n in (2, 3)
    ]


def test_proc_mount_exception1():
    with pytest.raises(SkipException) as e:
        ProcMounts(context_wrap(PROC_EXCEPTION1))
    assert 'Empty content' in str(e)


def test_proc_mount_exception2():
    with pytest.raises(ParseException) as e:
        ProcMounts(context_wrap(PROC_EXCEPTION2))
    assert "Input for mount must contain '/' mount point." in str(e)


def test_proc_mount_exception3():
    with pytest.raises(ParseException) as pe:
        ProcMounts(context_wrap(PROCMOUNT_ERR_DATA))
    assert 'Unable to parse' in str(pe.value)


def test_doc_examples():
    env = {
            'mnt_info': Mount(context_wrap(MOUNT_DOC)),
            'proc_mnt_info': ProcMounts(context_wrap(PROC_MOUNT_DOC)),
            'mounts': Mount(context_wrap(MOUNT_DOC))

          }
    failed, total = doctest.testmod(mount, globs=env)
    assert failed == 0



import doctest

from insights.parsers import ndctl_list
from insights.parsers.ndctl_list import NdctlListNi
from insights.parsers.tests import test_empty_skip
from insights.tests import context_wrap

NDCTL_OUTPUT = """
[
    {
        "dev":"namespace1.0",
        "mode":"fsdax",
        "map":"mem",
        "size":811746721792,
        "uuid":"6a7d93f5-60c4-461b-8d19-0409bd323a94",
        "sector_size":512,
        "align":2097152,
        "blockdev":"pmem1"
    },
    {
        "dev":"namespace1.1",
        "mode":"raw",
        "size":0,
        "uuid":"00000000-0000-0000-0000-000000000000",
        "sector_size":512,
        "state":"disabled"
    },
    {
        "dev":"namespace0.0",
        "mode":"raw",
        "size":0,
        "uuid":"00000000-0000-0000-0000-000000000000",
        "sector_size":512,
        "state":"disabled"
    }
]
""".strip()


def test_ndctl_list_doc_examples():
    env = {
        'ndctl_list': NdctlListNi(context_wrap(NDCTL_OUTPUT))
    }
    failed, total = doctest.testmod(ndctl_list, globs=env)
    assert failed == 0


def test_get_dev_attr():
    ndctl = NdctlListNi(context_wrap(NDCTL_OUTPUT))
    assert ndctl.blockdev_list == ['pmem1']
    assert 'map' in ndctl.get_blockdev('pmem1')
    assert ndctl.get_blockdev('pmem1').get('map') == 'mem'
    assert ndctl.get_blockdev('pmem2') == {}


def test_empty():
    assert 'Empty output.' in test_empty_skip(NdctlListNi)



import pytest
from insights.parsers import mdstat
from insights.tests import context_wrap

MDSTAT_TEST_1 = """
Personalities : [raid1] [raid6] [raid5] [raid4]
md_d0 : active raid5 sde1[0] sdf1[4] sdb1[5] sdd1[2] cciss/c0d0p1[6] sdc1[1]
      1250241792 blocks super 1.2 level 5, 64k chunk, algorithm 2 [5/5] [UUUUUU]
      bitmap: 0/10 pages [0KB], 16384KB chunk

unused devices: <none>
""".strip()

MDSTAT_RESULT_1 = {
    "personalities": ["raid1", "raid6", "raid5", "raid4"],
    "components": [
        {"device_name": "md_d0", "active": True, "auto_read_only": False, "raid": "raid5", "component_name": "sde1", "device_flag": '', "role": 0, "up": True, 'blocks': 1250241792, 'level': 5, 'chunk': '64k', 'algorithm': 2},
        {"device_name": "md_d0", "active": True, "auto_read_only": False, "raid": "raid5", "component_name": "sdf1", "device_flag": '', "role": 4, "up": True, 'blocks': 1250241792, 'level': 5, 'chunk': '64k', 'algorithm': 2},
        {"device_name": "md_d0", "active": True, "auto_read_only": False, "raid": "raid5", "component_name": "sdb1", "device_flag": '', "role": 5, "up": True, 'blocks': 1250241792, 'level': 5, 'chunk': '64k', 'algorithm': 2},
        {"device_name": "md_d0", "active": True, "auto_read_only": False, "raid": "raid5", "component_name": "sdd1", "device_flag": '', "role": 2, "up": True, 'blocks': 1250241792, 'level': 5, 'chunk': '64k', 'algorithm': 2},
        {"device_name": "md_d0", "active": True, "auto_read_only": False, "raid": "raid5", "component_name": "cciss/c0d0p1", "device_flag": '', "role": 6, "up": True, 'blocks': 1250241792, 'level': 5, 'chunk': '64k', 'algorithm': 2},
        {"device_name": "md_d0", "active": True, "auto_read_only": False, "raid": "raid5", "component_name": "sdc1", "device_flag": '', "role": 1, "up": True, 'blocks': 1250241792, 'level': 5, 'chunk': '64k', 'algorithm': 2}
    ]
}

MDSTAT_TEST_2 = """
Personalities : [raid1] [raid6] [raid5] [raid4]
md1 : active raid1 sdb2[1] sda2[0]
      136448 blocks [2/2] [UU]

md2 : active raid1 sdb3[1] sda3[0]
      129596288 blocks [2/2] [U_]

md0 : active raid1 sdb1[1](F) sda1[0]
      16787776 blocks [2/2] [_U]

unused devices: <none>
""".strip()

MDSTAT_RESULT_2 = {
    "personalities": ["raid1", "raid6", "raid5", "raid4"],
    "components": [
        {"device_name": "md1", "active": True, "auto_read_only": False, "raid": "raid1", "component_name": "sdb2", "device_flag": '', "role": 1, "up": True, 'blocks': 136448},
        {"device_name": "md1", "active": True, "auto_read_only": False, "raid": "raid1", "component_name": "sda2", "device_flag": '', "role": 0, "up": True, 'blocks': 136448},
        {"device_name": "md2", "active": True, "auto_read_only": False, "raid": "raid1", "component_name": "sdb3", "device_flag": '', "role": 1, "up": True, 'blocks': 129596288},
        {"device_name": "md2", "active": True, "auto_read_only": False, "raid": "raid1", "component_name": "sda3", "device_flag": '', "role": 0, "up": False, 'blocks': 129596288},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid1", "component_name": "sdb1", "device_flag": 'F', "role": 1, "up": False, 'blocks': 16787776},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid1", "component_name": "sda1", "device_flag": '', "role": 0, "up": True, 'blocks': 16787776}
    ]
}

MDSTAT_TEST_3 = """
Personalities : [linear] [raid0] [raid1]
unused devices: <none>
""".strip()

MDSTAT_RESULT_3 = {"personalities": ["linear", "raid0", "raid1"], "components": []}

MDSTAT_TEST_4 = """
Personalities : [linear] [raid0] [raid1]
md0 : inactive sdb[1](S) sda[0](S)
      6306 blocks super external:imsm<Paste>

unused devices: <none>
""".strip()

MDSTAT_RESULT_4 = {
    "personalities": ["linear", "raid0", "raid1"],
    "components": [
        {'device_flag': 'S', 'raid': None, 'device_name': 'md0', 'role': 1, 'active': False, 'auto_read_only': False, 'component_name': 'sdb', 'blocks': 6306},
        {'device_flag': 'S', 'raid': None, 'device_name': 'md0', 'role': 0, 'active': False, 'auto_read_only': False, 'component_name': 'sda', 'blocks': 6306}
    ]
}

PERSONALITIES_TEST = "Personalities : [linear] [raid0] [raid1] [raid5] [raid4] [raid6]\n"

PERSONALITIES_FAIL = [
    "Some stupid line.",
    "Personalities [raid1]",
    "Personalities : [raid1] some trash"
]

MD_TEST_1 = "md0 : active raid6 sdf1[0] sde1[1] sdd1[2](F) sdc1[3] sdb1[4](F) sda1[5] hdb1[6]"
MD_RESULT_1 = [
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "sdf1", "role": 0, "device_flag": ''},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "sde1", "role": 1, "device_flag": ''},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "sdd1", "role": 2, "device_flag": 'F'},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "sdc1", "role": 3, "device_flag": ''},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "sdb1", "role": 4, "device_flag": 'F'},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "sda1", "role": 5, "device_flag": ''},
        {"device_name": "md0", "active": True, "auto_read_only": False, "raid": "raid6", "component_name": "hdb1", "role": 6, "device_flag": ''}
]

MD_TEST_2 = "md0 : active (auto-read-only) raid6 sdf1[0] sde1[1] sdd1[2](S)"
MD_RESULT_2 = [
        {"device_name": "md0", "active": True, "auto_read_only": True, "raid": "raid6", "component_name": "sdf1", "role": 0, "device_flag": ''},
        {"device_name": "md0", "active": True, "auto_read_only": True, "raid": "raid6", "component_name": "sde1", "role": 1, "device_flag": ''},
        {"device_name": "md0", "active": True, "auto_read_only": True, "raid": "raid6", "component_name": "sdd1", "role": 2, "device_flag": 'S'}
]

MD_FAIL = [
    "what? : active raid5 sdh1[6] sdg1[4] sdf1[3] sde1[2] sdd1[1] sdc1[0]",
    "md124  active raid5 sdh1[6] sdg1[4] sdf1[3] sde1[2] sdd1[1] sdc1[0]",
    "md124 : raid5 sdh1[6] sdg1[4] sdf1[3] sde1[2] sdd1[1] sdc1[0]",
    "md124 : junk raid5 sdh1[6] sdg1[4] sdf1[3] sde1[2] sdd1[1] sdc1[0]",
    "md124 : active raid5 sdh16 sdg1[4] sdf1[3] sde1[2] sdd1[1] sdc1[0]",
    "md124 : active raid5 sdh1[6] sdg1[4]junk sdf1[3] sde1[2] sdd1[1] sdc1[0]"
    "md124 : active raid5 sdh1[6] sdg1[4]junk sdf1[3] sde1[2] sdd1[1] sdc1[0]"
]

UPSTRING_TEST_1 = "488383936 blocks [6/4] [_UUU_U]"
UPSTRING_TEST_2 = "1318680576 blocks level 5, 1024k chunk, algorithm 2 [10/10] [UUUUUUUUUU]"
UPSTRING_TEST_3 = "[==>..................]  recovery = 12.6% (37043392/292945152) finish=127.5min speed=33440K/sec"


def test_parse_personalities():
    result = mdstat.parse_personalities(PERSONALITIES_TEST)
    assert ["linear", "raid0", "raid1", "raid5", "raid4", "raid6"] == result

    for line in PERSONALITIES_FAIL:
        with pytest.raises(AssertionError):
            mdstat.parse_personalities(line)


def test_parse_array_start():
    result = mdstat.parse_array_start(MD_TEST_1)
    assert MD_RESULT_1 == result

    result = mdstat.parse_array_start(MD_TEST_2)
    assert MD_RESULT_2 == result

    for md_line in MD_FAIL:
        with pytest.raises(AssertionError):
            mdstat.parse_array_start(md_line)


def test_parse_upstring():
    result = mdstat.parse_upstring(UPSTRING_TEST_1)
    assert '_UUU_U' == result

    result = mdstat.parse_upstring(UPSTRING_TEST_2)
    assert 'UUUUUUUUUU' == result

    result = mdstat.parse_upstring(UPSTRING_TEST_3)
    assert result is None


def test_apply_upstring():
    test_dict = [{}, {}, {}, {}]
    mdstat.apply_upstring('U_U_', test_dict)
    assert test_dict[0]['up']
    assert not test_dict[1]['up']
    assert test_dict[2]['up']
    assert not test_dict[3]['up']

    with pytest.raises(AssertionError):
        mdstat.apply_upstring('U?_U', test_dict)

    with pytest.raises(AssertionError):
        mdstat.apply_upstring('U_U', test_dict)


def test_mdstat_construction():
    def compare_mdstat_data(test_data, parser_obj):
        """
        Because the dictionaries are huge and comparing them usually
        ends up with too many differences, it's better to compare them
        part by part
        """
        assert test_data['personalities'] == parser_obj.data['personalities']
        for testdata, objdata in zip(test_data['components'], parser_obj.data['components']):
            assert testdata == objdata

    mdstat_obj = mdstat.Mdstat(context_wrap(MDSTAT_TEST_1))
    compare_mdstat_data(MDSTAT_RESULT_1, mdstat_obj)

    assert len(mdstat_obj.mds) == 1
    assert sorted(mdstat_obj.mds.keys()) == ['md_d0']
    md_d0 = mdstat_obj.mds['md_d0']
    assert md_d0['name'] == 'md_d0'
    assert md_d0['active']
    assert md_d0['raid'] == 'raid5'
    assert len(md_d0['devices']) == 6
    assert sorted(md_d0['devices'][0].keys()) == sorted(['component_name', 'role', 'up'])
    assert md_d0['devices'][0]['component_name'] == 'sde1'
    assert md_d0['devices'][0]['role'] == 0
    assert md_d0['devices'][0]['up']

    # State line attributes
    #       1250241792 blocks super 1.2 level 5, 64k chunk, algorithm 2 [5/5] [UUUUUU]
    assert md_d0['blocks'] == 1250241792
    assert md_d0['level'] == 5
    assert md_d0['chunk'] == '64k'
    assert md_d0['algorithm'] == 2

    result = mdstat.Mdstat(context_wrap(MDSTAT_TEST_2))
    compare_mdstat_data(MDSTAT_RESULT_2, result)

    mdstat_obj = mdstat.Mdstat(context_wrap(MDSTAT_TEST_3))
    compare_mdstat_data(MDSTAT_RESULT_3, mdstat_obj)

    result = mdstat.Mdstat(context_wrap(MDSTAT_TEST_4))
    compare_mdstat_data(MDSTAT_RESULT_4, result)



import doctest
import pytest

from insights.parsers import engine_db_query, ParseException
from insights.parsers.tests import test_empty_skip
from insights.tests import context_wrap


OUTPUT = """
{
  "id_host": "None",
  "when": "2020-06-21 12:45:59",
  "time": "0.00263094902039",
  "name": "None",
  "description": "None",
  "type": "None",
  "kb": "None",
  "bugzilla": "None",
  "file": "",
  "path": "None",
  "id": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
  "hash": "d41d8cd98f00b204e9800998ecf8427e",
  "result": [
    {
      "vds_name": "hosto",
      "rpm_version": "vdsm-4.30.40-1.el7ev"
    }
  ]
}
""".strip()

OUTPUT_2 = """
{
  "id_host": "None",
  "when": "2020-06-21 12:45:59",
  "time": "0.00263094902039",
  "name": "None",
  "description": "None",
  "type": "None",
  "kb": "None",
  "bugzilla": "None",
  "file": "",
  "path": "None",
  "id": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
  "hash": "d41d8cd98f00b204e9800998ecf8427e",
  "result": [
    {
      "vds_name": "hosto",
      "rpm_version": "vdsm-4.40.20-33.git1b7dedcf3.fc30"
    },
    {
      "vds_name": "hosto2",
      "rpm_version": "vdsm-4.40.13-38.gite9bae3c68.fc30"
    }
  ]
}
""".strip()

ERROR = """

            SELECT
                row_to_json(t)
            FROM
               (SELECT vs.vds_name, rpm_version FROM vds_dynamic vd, vds_static vs WHERE vd.vds_id = vs.vds_id;) t

Traceback (most recent call last):
  File "/usr/lib/python2.7/site-packages/engine_db_query/__init__.py", line 337, in query_return_json
    query
SyntaxError: syntax error at or near ";"
LINE 5: ...ds_dynamic vd, vds_static vs WHERE vd.vds_id = vs.vds_id;) t
                                                                   ^

Traceback (most recent call last):
  File "/usr/bin/engine-db-query", line 281, in <module>
    sys.exit(main())
  File "/usr/bin/engine-db-query", line 273, in main
    knowledge_base=args.kb_url
  File "/usr/lib/python2.7/site-packages/engine_db_query/__init__.py", line 213, in execute
    knowledge_base=knowledge_base
  File "/usr/lib/python2.7/site-packages/engine_db_query/__init__.py", line 348, in query_return_json
    ret = cursor.fetchall()
psycopg2.ProgrammingError: no results to fetch
""".strip()


def test_edbq():
    output = engine_db_query.EngineDBQueryVDSMversion(context_wrap(OUTPUT))
    assert output.get('id', None) == 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
    assert output.result[0].get('rpm_version') == 'vdsm-4.30.40-1.el7ev'

    # for multiple hosts
    output = engine_db_query.EngineDBQueryVDSMversion(context_wrap(OUTPUT_2))
    assert output.result == [{'vds_name': 'hosto', 'rpm_version': 'vdsm-4.40.20-33.git1b7dedcf3.fc30'}, {'vds_name': 'hosto2', 'rpm_version': 'vdsm-4.40.13-38.gite9bae3c68.fc30'}]

    # No content
    assert 'Empty output.' in test_empty_skip(engine_db_query.EngineDBQueryVDSMversion)

    # Error
    with pytest.raises(ParseException) as e:
        engine_db_query.EngineDBQueryVDSMversion(context_wrap(ERROR))
    assert "couldn't parse json." in str(e)


def test_doc_examples():
    env = {
        'output': engine_db_query.EngineDBQueryVDSMversion(context_wrap(OUTPUT))
    }
    failed, total = doctest.testmod(engine_db_query, globs=env)
    assert failed == 0



import doctest

from insights.parsers import scheduler
from insights.tests import context_wrap

SDA_SCHEDULER = '''
noop deadline [cfq]
'''.strip()

SDB_SCHEDULER = '''
noop [deadline] cfq
'''.strip()

VDC_SCHEDULER = '''
[noop] deadline cfq
'''.strip()

SDA_PATH = "/sys/block/sda/queue/scheduler"
SDB_PATH = "/sys/block/sdb/queue/scheduler"
VDC_PATH = "/sys/block/vdc/queue/scheduler"


def test_scheduler_cfq():
    r = scheduler.Scheduler(context_wrap(SDA_SCHEDULER, SDA_PATH))
    assert r.data["sda"] == '[cfq]'


def test_scheduler_deadline():
    r = scheduler.Scheduler(context_wrap(SDB_SCHEDULER, SDB_PATH))
    assert r.data["sdb"] == '[deadline]'


def test_scheduler_noop():
    r = scheduler.Scheduler(context_wrap(VDC_SCHEDULER, VDC_PATH))
    assert r.data["vdc"] == '[noop]'


def test_schedulers_defaults():
    r = scheduler.Scheduler(context_wrap('[none] mq-deadline kyber bfq',
                                         '/sys/block/nvme0n1/queue/scheduler'))
    assert r.device == 'nvme0n1'
    assert r.schedulers == ['none', 'mq-deadline', 'kyber', 'bfq']
    assert r.active_scheduler == 'none'

    # RHEL 6
    r = scheduler.Scheduler(context_wrap('noop anticipatory deadline [cfq]',
                                         '/sys/block/vda/queue/scheduler'))
    assert r.device == 'vda'
    assert r.schedulers == ['noop', 'anticipatory', 'deadline', 'cfq']
    assert r.active_scheduler == 'cfq'

    # RHEL 7
    r = scheduler.Scheduler(context_wrap('[mq-deadline] kyber none',
                                         '/sys/block/vda/queue/scheduler'))
    assert r.device == 'vda'
    assert r.schedulers == ['mq-deadline', 'kyber', 'none']
    assert r.active_scheduler == 'mq-deadline'

    # RHEL 8
    r = scheduler.Scheduler(context_wrap('[mq-deadline] kyber bfq none',
                                         '/sys/block/vda/queue/scheduler'))
    assert r.device == 'vda'
    assert r.schedulers == ['mq-deadline', 'kyber', 'bfq', 'none']
    assert r.active_scheduler == 'mq-deadline'


def test_none():
    r = scheduler.Scheduler(context_wrap('none',
                                         '/sys/block/vda/queue/scheduler'))
    assert r.device == 'vda'
    assert r.schedulers == ['none']
    assert r.active_scheduler is None
    assert r.data == {}


def test_docs():
    env = {
        'scheduler_obj': scheduler.Scheduler(context_wrap(SDA_SCHEDULER, SDA_PATH))
    }
    failed, total = doctest.testmod(scheduler, globs=env)
    assert failed == 0



"""
LsRunSystemdGenerator - command ``ls -lan /run/systemd/generator``
==================================================================

The ``ls -lan /run/systemd/generator`` command provides information for only
the ``/run/systemd/generator`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample directory list::

    total 28
    drwxr-xr-x.  6 0 0 260 Aug  5 07:35 .
    drwxr-xr-x. 18 0 0 440 Aug  5 07:35 ..
    -rw-r--r--.  1 0 0 254 Aug  5 07:35 boot.mount
    -rw-r--r--.  1 0 0 259 Aug  5 07:35 boot\x2dfake.mount
    -rw-r--r--.  1 0 0 176 Aug  5 07:35 dev-mapper-rhel\x2dswap.swap
    drwxr-xr-x.  2 0 0 100 Aug  5 07:35 local-fs.target.requires
    -rw-r--r--.  1 0 0 217 Aug  5 07:35 -.mount
    drwxr-xr-x.  2 0 0  60 Aug  5 07:35 nfs-server.service.d
    drwxr-xr-x.  2 0 0 100 Aug  5 07:35 remote-fs.target.requires
    -rw-r--r--.  1 0 0 261 Aug  5 07:35 root-mnt_nfs3.mount
    -rw-r--r--.  1 0 0 261 Aug  5 07:35 root-mnt\x2dnfs1.mount
    -rw-r--r--.  1 0 0 261 Aug  5 07:35 root-mnt\x2dnfs2.mount
    drwxr-xr-x.  2 0 0  60 Aug  5 07:35 swap.target.requires

Examples:

    >>> ls.files_of("/run/systemd/generator") == ['boot.mount', 'boot-fake.mount', 'dev-mapper-rhel-swap.swap', '-.mount', 'root-mnt_nfs3.mount', 'root-mnt-nfs1.mount', 'root-mnt-nfs2.mount']
    True
    >>> ls.dir_entry("/run/systemd/generator", '-.mount')['perms']
    'rw-r--r--.'
"""

from insights import parser, CommandParser, FileListing
from insights.specs import Specs


@parser(Specs.ls_run_systemd_generator)
class LsRunSystemdGenerator(CommandParser, FileListing):
    """Parses output of ``ls -lan /run/systemd/generator`` command."""
    pass



"""
Pluggable Authentication Module configuration
=============================================

This module provides parsing for PAM configuration files.
``PamConf`` is a parser for ``/etc/pam.conf`` files. Sample input is provided
in the examples.

PamConf - file ``/etc/pam.conf``
--------------------------------

Sample file data::

    #%PAM-1.0
    vsftpd      auth        required    pam_securetty.so
    vsftpd      auth        requisite   pam_unix.so nullok
    vsftpd      auth        sufficient  pam_nologin.so
    vsftpd      account     optional    pam_unix.so
    other       password    include     pam_cracklib.so retry=3 logging=verbose
    other       password    required    pam_unix.so shadow nullok use_authtok
    other       session     required    pam_unix.so


Examples:
    >>> type(pam_conf)
    <class 'insights.parsers.pam.PamConf'>
    >>> len(pam_conf)
    7
    >>> pam_conf[0].service
    'vsftpd'
    >>> pam_conf[0].interface
    'auth'
    >>> pam_conf[0].control_flags
    [ControlFlag(flag='required', value=None)]
    >>> pam_conf[0].module_name
    'pam_securetty.so'
    >>> pam_conf[0].module_args is None
    True
    >>> pam_conf.file_path
    '/etc/pam.conf'

PamDConf - used for specific PAM configuration files
----------------------------------------------------

``PamDConf`` is a base class for the creation of parsers for ``/etc/pam.d``
service specific configuration files.

Sample file from ``/etc/pam.d/sshd``::

    #%PAM-1.0
    auth       required     pam_sepermit.so
    auth       substack     password-auth
    auth       include      postlogin
    # Used with polkit to reauthorize users in remote sessions
    -auth      optional     pam_reauthorize.so prepare
    account    required     pam_nologin.so
    account    include      password-auth
    password   include      password-auth
    # pam_selinux.so close should be the first session rule
    session    required     pam_selinux.so close
    session    required     pam_loginuid.so
    # pam_selinux.so open should only be followed by sessions to be executed in the user context
    session    required     pam_selinux.so open env_params
    session    required     pam_namespace.so
    session    optional     pam_keyinit.so force revoke
    session    include      password-auth
    session    include      postlogin
    # Used with polkit to reauthorize users in remote sessions
    -session   optional     pam_reauthorize.so prepare

Examples:

    >>> type(pamd_conf)
    <class 'insights.parsers.pam.PamDConf'>
    >>> len(pamd_conf)
    15
    >>> pamd_conf[0]._errors == [] # No errors in parsing
    True
    >>> pamd_conf[0].service
    'sshd'
    >>> pamd_conf[0].interface
    'auth'
    >>> pamd_conf[0].control_flags
    [ControlFlag(flag='required', value=None)]
    >>> pamd_conf[0].module_name
    'pam_sepermit.so'
    >>> pamd_conf[0].module_args is None
    True
    >>> pamd_conf.file_path
    '/etc/pam.d/sshd'
    >>> pamd_conf[3].module_name
    'pam_reauthorize.so'
    >>> pamd_conf[3].ignored_if_module_not_found
    True

Normal use of the ``PamDConf`` class is to subclass it for a parser.  In
``insights/specs/default.py``::

    pam_sshd = simple_file("etc/pam.d/sshd")

In the parser module (e.g. ``insights/parsers/pam_sshd.py``)::

    from insights import parser
    from insights.parsers.pam import PamDConf
    from insights.specs import Specs

    @parser(Specs.pam_sshd)
    class PamSSHD(PamDConf):
        pass

References:
    http://www.linux-pam.org/Linux-PAM-html/Linux-PAM_SAG.html
"""

from collections import namedtuple
from .. import Parser, get_active_lines, parser
from ..parsers import unsplit_lines, keyword_search, optlist_to_dict

import re
from insights.specs import Specs


class PamConfEntry(object):
    """Contains information from one PAM configuration line.

    Parses a single line of either a ``/etc/pam.conf`` file or
    a service specific ``/etc/pam.d`` conf file. The difference
    is that for ``/etc/pam.conf``, the service name is the first
    column of the input line. If a service specific conf file
    then the service name is not present in the line and must
    be provided as the ``service`` parameter as well as setting
    the ``pamd_conf`` to True.

    Parameters:
        line (str): One line of the pam conf info.
        pamd_config (boolean): If this is set to False then
            ``line`` will be parsed as a line from the ``etc/pam.conf`` file,
            if ``True`` then the line will be parsed as a line from a service
            specific ``etc/pam.d/`` conf file. Default is ``True``.
        service (str): If ``pamd_conf`` is ``True`` then the name of the service file
            must be provided since it is not present in `line`.

    Attributes:
        service (str): The service name (taken from the line or from the
            file name if not parsing ``pam.conf``)
        interface (str): The *type* clause - should be one of ``'account'``,
            ``'auth'``, ``'password'`` or ``'session'.  If the line was
            invalid this is set to ``None``.
        ignored_if_module_not_found (bool): If the *type* clause is preceded
            by ``'-'``, then this is set to True and it indicates that PAM
            would skip this line rather than reporting an error if the given
            module is not found.
        control_flags (list): A list of ControlFlag named tuples.  If the
            control flag was one of ``'required'``, ``'requisite'``,
            ``'sufficient'``, ``'optional'``, ``'include'``, or
            ``'substack'``, then this is the only flag in the list and its
            value is set to ``True``.  If the control flag started with ``[``,
            then the list inside the square brackets is interpreted as a list
            of key=value tuples.
        _control_raw (str): the raw control flag string before parsing, for
            reference.
        module_name (str): the PAM module name (including the '.so')
        module_args (str): the PAM module arguments, if any.  This is not
            parsed.
        _full_line (str): The original line in the PAM configuration.
        _errors (list): A list of parsing errors detected in this line.

    Examples:
        >>> pam_conf_line = 'vsftpd      auth        requisite   pam_unix.so nullok'
        >>> entry = PamConfEntry(pam_conf_line)
        >>> entry.service
        'vsftpd'
        >>> entry.control_flags[0].flag
        'requisite'
        >>> entry.module_args
        'nullok'
        >>> pamd_conf_line = '''
        ... auth        [success=2 default=ok]  pam_debug.so auth=perm_denied cred=success
        ... '''.strip()
        >>> entry = PamConfEntry(pamd_conf_line, pamd_conf=True, service='vsftpd')
        >>> entry.service
        'vsftpd'
        >>> entry.control_flags
        [ControlFlag(flag='success', value='2'), ControlFlag(flag='default', value='ok')]
        >>> entry.module_args
        'auth=perm_denied cred=success'

    Raises:
        ValueError: If `pamd_conf` is True and `service` name is not provided,
            or if the line doesn't contain any module information.

    .. describe:: ControlFlag

        A named tuple with the 'flag' and 'value' properties, used to store
        information about the control flags in a PAM configuration line.
    """

    ControlFlag = namedtuple('ControlFlag', 'flag, value')

    type_re = r'(?P<type>-?(?:account|auth|password|session))'
    control_re = r'(?P<control>required|requisite|sufficient|optional|' +\
        r'include|substack|\[\w+=\w+(?:\s+\w+=\w+)*\])'
    mod_path_re = r'(?P<module>[\w.-]+)'
    mod_args_re = r'(?P<mod_args>\S.*)'
    line_re = r'\s+'.join([type_re, control_re, mod_path_re]) + \
        r'(?:\s+' + mod_args_re + r')?'
    line_rex = re.compile(line_re)

    def __init__(self, line, pamd_conf=False, service=None):
        # If not pam.d file then service is first column in
        # line for pam.conf file. Otherwise service must be parameter.
        # print "PamConfEntry(line='{l}', pamd_conf={p}, service={s})".format(l=line, p=pamd_conf, s=service)
        self._full_line = line
        self._errors = []
        self.service = service
        self.interface = None
        self.ignored_if_module_not_found = None
        self._control_raw = ''
        self.control_flags = []
        self.module_name = None
        self.module_args = None
        if not pamd_conf:
            # Because this is called using get_active_lines() and
            # line_unsplit(), we cannot get a line that does not at least have
            # one token on it.
            self.service, line = line.split(None, 1)
        elif service is None:
            # The only valid situation to raise an error - the implementation
            # has called us with pamd_conf = true so we expect a service to
            # be given as a parameter.
            raise ValueError('Service name must be provided for pam.d conf file')

        match = self.line_rex.search(line)
        if match:
            # Type can have a '-' in front, if so line is ignored if module
            # cannot be found.
            self._type_raw = match.group('type')
            self.ignored_if_module_not_found = self._type_raw[0] == '-'
            self.interface = self._type_raw[1:] if self.ignored_if_module_not_found else self._type_raw
            # Parse control token, either regular word or [key=val...]
            self._control_raw = match.group('control')
            if self._control_raw.startswith('['):
                self.control_flags = []
                # Regex assures that it ends with ]
                for group in self._control_raw[1:-1].split(None):
                    # We could do a match: val is constrained to a list and
                    # action is 'ignore', 'bad', 'die', 'ok', done', 'reset'
                    # or N where N is an unsigned integer.  But keep it simple
                    # for now.
                    # Regex makes sure that each group contains an '=' though.
                    val, action = group.split('=', 1)
                    self.control_flags.append(self.ControlFlag(val, action))
            else:
                self.control_flags = [self.ControlFlag(self._control_raw, None)]
            self.module_name = match.group('module')
            self.module_args = match.group('mod_args') if 'mod_args' in match.groupdict() else None
            self.module_args_dict = (
                optlist_to_dict(self.module_args, opt_sep=' ')
                if self.module_args is not None
                else {}
            )
        else:
            # Line not valid - report error
            self._errors.append("Cannot parse line '{l}' as a valid pam.d entry".format(l=self._full_line))

    def __repr__(self):
        return "<PamConfEntry for {svc}: {typ} {ctl} {name}{args}>".format(
            svc=self.service, typ=self._type_raw, ctl=self._control_raw,
            name=self.module_name, args=(
                ' ' + self.module_args if self.module_args else ''
            )
        )


class PamDConf(Parser):
    """Base class for parsing files in ``/etc/pam.d``

    Derive from this class for parsers of files in
    the ``/etc/pam.d`` directory.  Parses each line of the conf
    file into a list of PamConfEntry.  Configuration file format is::

        module_interface    control_flag    module_name module_arguments

    Sample input::
        >>> pam_sshd = '''
        ... auth        required    pam_securetty.so
        ... auth        requisite   pam_unix.so nullok
        ... auth        sufficient  pam_nologin.so
        ... auth        [success=2 default=ok]  pam_debug.so auth=perm_denied cred=success
        ... account     optional    pam_unix.so
        ... password    include     pam_cracklib.so retry=3 logging=verbose
        ... password    required    pam_unix.so shadow nullok use_authtok
        ... '''
        >>> from insights.tests import context_wrap
        >>> class YourPamDConf(PamDConf):  # A trivial example
        ...     pass
        >>> conf = YourPamDConf(context_wrap(pam_sshd, path='/etc/pam.d/sshd'))

    The `service` property of each PamConfEntry is set to the complete path
    name of the PAM config file.

    Attributes:
        data (list): List containing a PamConfEntry object for each line of
            the conf file in the same order as lines appear in the file.

    Examples:
        >>> conf[0].module_name  # Can be used like a list of objects
        'pam_securetty.so'
        >>> account_rows = list(conf.search(interface='account'))
        >>> len(account_rows)
        1
        >>> account_rows[0].interface
        'account'
        >>> account_rows[0].module_name
        'pam_unix.so'
        >>> account_rows[0].control_flags
        [ControlFlag(flag='optional', value=None)]
    """
    def parse_content(self, content):
        self.data = []
        # Need unsplit_lines for handling \ line continuations
        for line in get_active_lines(unsplit_lines(content)):
            self.data.append(PamConfEntry(line, pamd_conf=True, service=self.file_name))

    def __iter__(self):
        """(iterable): Iterate through the list of rules in the file"""
        for entry in self.data:
            yield entry

    def __getitem__(self, ndx):
        """(PamConfEntry): Return the entry data for the given line number"""
        return self.data[ndx]

    def __len__(self):
        """(int): Return the number of entries read from the file"""
        return len(self.data)

    def search(self, **kwargs):
        """
        Search the pam.d configuration file by keyword.  This is provided by
        the :func:`insights.parsers.keyword_search` function - see its
        documentation for more information.

        Searching on the list of PAM configuration entries is exactly like
        they were dictionaries instead of objects with properties.  In
        addition, the 'control_flags' property becomes a dictionary of
        keywords and values, so that 'control_flags__contains' allows
        searching for a particular control flag.

        Returns:
            (list): A list of PamConfEntry objects that match the given
            search criteria.
        """
        # Because keyword_search takes dicts, and we have objects with complex
        # properties, we convert them to a list of dicts.
        # First, store the things we're going to convert.
        prop_keys = ('service', 'interface', 'module_name', 'module_args')
        search = []
        # Can't find a neat way to do this as a comprehension, so it's back
        # to loops.
        for obj in self.data:
            row = {'entry_obj': obj}
            for key in prop_keys:
                row[key] = getattr(obj, key)
            # For hysterical reasons, module_args can contain None if not
            # defined, but that's non-iterable: replace with ''
            if row['module_args'] is None:
                row['module_args'] = ''
            # Convert control_flags down to key/value dictionary
            flags = {}
            for cf in obj.control_flags:
                flags[cf.flag] = cf.value
            row['control_flags'] = flags
            search.append(row)
        # Now get the result, and just return the entry objects
        found = []
        for r in keyword_search(search, **kwargs):
            found.append(r['entry_obj'])
        return found


@parser(Specs.pam_conf)
class PamConf(PamDConf):
    """Base class for parsing pam config file ``/etc/pam.conf``.

    Based on the PamDConf parser class, but the service must be given as
    the first element of the line, rather than assumed from the file name.
    """
    def parse_content(self, content):
        self.data = []
        for line in get_active_lines(content):
            self.data.append(PamConfEntry(line))



import os
import copy
import base64
import tempfile
import pkgutil
import hashlib
import insights.client.apps.ansible
from logging import getLogger
from insights.client.apps.ansible.playbook_verifier.contrib import gnupg
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel import yaml
from insights.client.constants import InsightsConstants as constants

__all__ = ("loadPlaybookYaml", "verify", "PlaybookVerificationError")

SIGKEY = 'insights_signature'
PUBLIC_KEY_FOLDER = pkgutil.get_data(insights.client.apps.ansible.__name__, 'playbook_verifier/public.gpg')    # Update this when we have the key generated
EXCLUDABLE_VARIABLES = ['hosts', 'vars']

logger = getLogger(__name__)

yaml = yaml.YAML(typ='rt')
yaml.indent(mapping=2, sequence=4, offset=2)
yaml.default_flow_style = False
yaml.preserve_quotes = True
yaml.width = 200


class PlaybookVerificationError(Exception):
    """
    Exception raised when playbook verification fails

    Attributes:
        playbook -- stringified playbook yaml from stdin
        message -- explanation of why verification failed
    """

    def __init__(self, message="PLAYBOOK VALIDATION FAILED"):
        self.message = message
        super(PlaybookVerificationError, self).__init__(self.message)

    def __str__(self):
        return self.message


def createSnippetHash(snippet):
    """
    Function that creates and returns a hash of the snippet given to the function.
        output: snippetHash (bytes)
    """
    snippetHash = hashlib.sha256()
    serializedSnippet = str(snippet).encode("UTF-8")
    snippetHash.update(serializedSnippet)

    return snippetHash.digest()


def getPublicKey(gpg):
        if not PUBLIC_KEY_FOLDER:
            raise PlaybookVerificationError(message="PUBLIC KEY IMPORT ERROR: Public key file not found")

        publicKey = PUBLIC_KEY_FOLDER
        importResults = gpg.import_keys(publicKey)
        if (importResults.count < 1):
            raise PlaybookVerificationError(message="PUBLIC KEY NOT IMPORTED: Public key import failed")

        return importResults


def excludeDynamicElements(snippet):
    exclusions = snippet['vars']['insights_signature_exclude'].split(',')

    for element in exclusions:
        element = element.split('/')

        # remove empty strings
        element = [string for string in element if string != '']

        if (len(element) == 1 and element[0] in EXCLUDABLE_VARIABLES):
            del snippet[element[0]]
        elif (len(element) == 2 and element[0] in EXCLUDABLE_VARIABLES):
            try:
                del snippet[element[0]][element[1]]
            except:
                raise PlaybookVerificationError(message='INVALID FIELD: the variable {0} defined in insights_signature_exclude does not exist.'.format(element))
        else:
            raise PlaybookVerificationError(message='INVALID EXCLUSION: the variable {0} is not a valid exclusion.'.format(element))

    return snippet


def executeVerification(snippet, encodedSignature):
    gpg = gnupg.GPG(gnupghome=constants.insights_core_lib_dir)
    snippetHash = createSnippetHash(snippet)

    decodedSignature = base64.b64decode(encodedSignature)

    # load public key
    getPublicKey(gpg)

    fd, fn = tempfile.mkstemp()
    os.write(fd, decodedSignature)
    os.close(fd)

    result = gpg.verify_data(fn, snippetHash)
    os.unlink(fn)

    return result


def verifyPlaybookSnippet(snippet):
    if ('vars' not in snippet.keys()):
        raise PlaybookVerificationError(message='VERIFICATION FAILED: Vars field not found')
    elif (snippet['vars'] is None):
        raise PlaybookVerificationError(message='VERIFICATION FAILED: Empty vars field')
    elif (SIGKEY not in snippet['vars']):
        raise PlaybookVerificationError(message='VERIFICATION FAILED: Signature not found')

    encodedSignature = snippet['vars'][SIGKEY]
    snippetCopy = copy.deepcopy(snippet)

    snippetCopy = excludeDynamicElements(snippetCopy)

    return executeVerification(snippetCopy, encodedSignature)


def verify(playbook, skipVerify=False):
    """
    Verify the signed playbook.

    Input: unverified playbook (dictionary format)
    Output: "verified" playbook (dictionary format)
    Error: Playbook Verification failure / Playbook Signature not found.
    """
    logger.info('Playbook Verification has started')

    if not skipVerify:
        for snippet in playbook:
            verified = verifyPlaybookSnippet(snippet)

            if not verified:
                raise PlaybookVerificationError(message="SIGNATURE NOT VALID: Template [name: {0}] has invalid signature".format(snippet['name']))

    logger.info('All templates successfully validated')
    return playbook


def loadPlaybookYaml(playbook):
    """
    Load playbook yaml using current yaml library implementation
        output: playbook yaml
    """
    return yaml.load(playbook)



"""
DesignateConf - file ``/etc/designate/designate.conf``
======================================================

This class provides parsing for the files:
    ``/var/lib/config-data/puppet-generated/designate/etc/designate/designate.conf``
    ``/etc/designate/designate.conf``

Sample input data is in the format::

    [DEFAULT]
    state_path=/var/lib/designate
    root_helper=sudo designate-rootwrap /etc/designate/rootwrap.conf
    debug=True
    log_dir=/var/log/designate

    [keystone_authtoken]
    www_authenticate_uri=http://localhost:5000
    project_name=service
    project_domain_name=Default

    [oslo_messaging_notifications]
    driver=messagingv2

See the ``IniConfigFile`` class for examples.
"""
from .. import IniConfigFile, parser, add_filter
from insights.specs import Specs

ADDITIONAL_FILTERS = [
    "[",
    "state_path",
    "root_helper",
    "debug",
    "log_dir",
    "www_authenticate_uri",
    "project_name",
    "project_domain_name",
    "driver"
]
add_filter(Specs.designate_conf, ADDITIONAL_FILTERS)


@parser(Specs.designate_conf)
class DesignateConf(IniConfigFile):
    """Class to parse file ``designate.conf``."""
    pass
